#ifndef __GURUMDDS_DCPSX__
#define __GURUMDDS_DCPSX__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h>

typedef struct _dds_StringProperty {
	char*	key;
	void*	value;
} dds_StringProperty;

/** The dds_SampleInfoEx can be used with dds_SampleInfoSeq APIs */
typedef struct {
	dds_SampleInfo info;
	struct {
		uint8_t prefix[12];
		uint32_t entityId;
	} src_guid;
	uint64_t seq;
	uint8_t keyhash[16];
} dds_SampleInfoEx;

typedef struct _dds_TypeDeliveryService dds_TypeDeliveryService;
/**
 * Get a value from dds_StringProperty
 *
 * @param props properties
 * @param key key
 *
 * @return value
 */
GURUMDDS_C_DLLAPI void* dds_StringProperty_get(dds_StringProperty* props, char* key);

/**
 * Check whether propery array has a given key
 *
 * @param props properties
 * @param key key
 *
 * @return key is contained or not
 */
GURUMDDS_C_DLLAPI bool dds_StringProperty_contains(dds_StringProperty* props, char* key);

/**
 * @file
 * GurumDDS Vendor Extension APIs
 */

/**
* Change middleware log level (overrides the value set by config)
*
* @param loglevel minimum log level from 0(VERBOSE) to 5(FATAL)
*/
GURUMDDS_C_DLLAPI void dds_DomainParticipantFactory_set_loglevel(int loglevel/*0(VERBOSE) ~ 5(FATAL)*/);

/**
 * Get a log level from middleware
 *
 * @return minimum log level from 0(VERBOSE) to 5(FATAL)
 */
GURUMDDS_C_DLLAPI int dds_DomainParticipantFactory_get_loglevel(void);

/**
 * @internal
 * Dump all of the contained entities by middleware to standard output
 */
GURUMDDS_C_DLLAPI void dds_DomainParticipantFactory_dump(void);

/**
 * @internal
 * Dump all of the contained entities by middleware to newly allocated string
 *
 * @param self participant factory object
 *
 * @return newly allocated string
 */
GURUMDDS_C_DLLAPI char* dds_DomainParticipantFactory_dump_as_string(dds_DomainParticipantFactory* self);

/**
 * Creates a new DomainParticipant object with given participant id
 *
 * The DomainParticipant signifies that the calling application intends to join the Domain identified by the domain_id argument
 *
 * @param self domainparticipantfactory
 * @param domain_id domain id
 * @param participant_id participant id
 * @param qos domainparticipant qos
 * @param a_listener domainparticipant listener
 * @param mask domainparticipant listener mask
 *
 * @return newly created DomainParticipant
 */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_DomainParticipantFactory_create_participant_w_id(dds_DomainParticipantFactory* self, dds_DomainId_t domainId, uint16_t participantId, const dds_DomainParticipantQos* qos, const dds_DomainParticipantListener* a_listener, dds_StatusMask mask);

/**
 * Creates a new DomainParticipant object with given properties
 *
 * The DomainParticipant signifies that the calling application intends to join the Domain identified by the domain_id argument
 *
 * @param self domainparticipantfactory
 * @param domain_id domain id
 * @param qos domainparticipant qos
 * @param a_listener domainparticipant listener
 * @param mask domainparticipant listener mask
 * @param props null-terminated property array
 *
 * @return newly created DomainParticipant
 */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_DomainParticipantFactory_create_participant_w_props(dds_DomainParticipantFactory* self, dds_DomainId_t domainId, const dds_DomainParticipantQos* qos, const dds_DomainParticipantListener* a_listener, dds_StatusMask mask, dds_StringProperty* props);

/**
 * Get a context variable from internal entity storage
 *
 * @param self entity
 * @param index storage index
 *
 * @return context variable
 */
GURUMDDS_C_DLLAPI void* dds_Entity_get_context(dds_Entity* self, int32_t index);

/**
 * Set a context variable into internal entity storage
 *
 * @param self entity
 * @param index storage index
 * @param context context variable
 *
 * @return true on succeed, false on out-of-storage
 */
GURUMDDS_C_DLLAPI bool dds_Entity_set_context(dds_Entity* self, int32_t index, void* context);

// Internal Qos Profile APIs -----------------------------------------------------------------------
// dds_ReturnCode_t dds_DomainParticipantFactoryQos_from_profile(const char* profile_path, dds_DomainParticipantFactoryQos* qos);
// dds_ReturnCode_t dds_DomainParticipantQos_from_profile(const char* profile_path, dds_DomainParticipantQos* qos);
// dds_ReturnCode_t dds_TopicQos_from_profile(const char* profile_path, dds_TopicQos* qos);
// dds_ReturnCode_t dds_PublisherQos_from_profile(const char* profile_path, dds_PublisherQos* qos);
// dds_ReturnCode_t dds_SubscriberQos_from_profile(const char* profile_path, dds_SubscriberQos* qos);
// dds_ReturnCode_t dds_DataWriterQos_from_profile(const char* profile_path, dds_DataWriterQos* qos);
// dds_ReturnCode_t dds_DataReaderQos_from_profile(const char* profile_path, dds_DataReaderQos* qos);

// dds_ReturnCode_t dds_DomainParticipantFactoryQos_create(dds_DomainParticipantFactoryQos* qos);
// dds_ReturnCode_t dds_DomainParticipantQos_create(dds_DomainParticipantQos* qos);
// dds_ReturnCode_t dds_PublisherQos_create(dds_PublisherQos* qos);
// dds_ReturnCode_t dds_SubscriberQos_create(dds_SubscriberQos* qos);
// dds_ReturnCode_t dds_TopicQos_create(dds_TopicQos* qos);
// dds_ReturnCode_t dds_DataWriterQos_create(dds_DataWriterQos* qos);
// dds_ReturnCode_t dds_DataReaderQos_create(dds_DataReaderQos* qos);

// // Listener Extensions -----------------------------------------------------------------------------
// getter: return associated listener context from entity
// setter: associate new listener context and returns previous one

/**
 * load listener context from domain participant
 *
 * @param self domain participant
 *
 * @return listener context
 */
GURUMDDS_C_DLLAPI void* dds_DomainParticipant_get_listener_context(dds_DomainParticipant* self);

/**
 * store listener context to domain participant
 *
 * @param self domain participant
 * @param context listener context
 *
 * @return previously stored listener context
 */
GURUMDDS_C_DLLAPI void* dds_DomainParticipant_set_listener_context(dds_DomainParticipant* self, void* context);

/**
 * load listener context from topic
 *
 * @param self topic
 *
 * @return listener context
 */
GURUMDDS_C_DLLAPI void* dds_Topic_get_listener_context(dds_Topic* self);

/**
 * store listener context to topic
 *
 * @param self topic
 * @param context listener context
 *
 * @return previously stored listener context
 */
GURUMDDS_C_DLLAPI void* dds_Topic_set_listener_context(dds_Topic* self, void* context);

/**
 * load listener context from publisher
 *
 * @param self publisher
 *
 * @return listener context
 */
GURUMDDS_C_DLLAPI void* dds_Publisher_get_listener_context(dds_Publisher* self);

/**
 * store listener context to publisher
 *
 * @param self publisher
 * @param context listener context
 *
 * @return previously stored listener context
 */
GURUMDDS_C_DLLAPI void* dds_Publisher_set_listener_context(dds_Publisher* self, void* context);

/**
 * load listener context from datawriter
 *
 * @param self datawriter
 *
 * @return listener context
 */
GURUMDDS_C_DLLAPI void* dds_DataWriter_get_listener_context(dds_DataWriter* self);

/**
 * store listener context to datawriter
 *
 * @param self datawriter
 * @param context listener context
 *
 * @return previously stored listener context
 */
GURUMDDS_C_DLLAPI void* dds_DataWriter_set_listener_context(dds_DataWriter* self, void* context);

/**
 * load listener context from subscriber
 *
 * @param self subscriber
 *
 * @return listener context
 */
GURUMDDS_C_DLLAPI void* dds_Subscriber_get_listener_context(dds_Subscriber* self);

/**
 * store listener context to subscriber
 *
 * @param self subscriber
 * @param context listener context
 *
 * @return previously stored listener context
 */
GURUMDDS_C_DLLAPI void* dds_Subscriber_set_listener_context(dds_Subscriber* self, void* context);

/**
 * load listener context from datareader
 *
 * @param self datareader
 *
 * @return listener context
 */
GURUMDDS_C_DLLAPI void* dds_DataReader_get_listener_context(dds_DataReader* self);

/**
 * store listener context to datareader
 *
 * @param self datareader
 * @param context listener context
 *
 * @return previously stored listener context
 */
GURUMDDS_C_DLLAPI void* dds_DataReader_set_listener_context(dds_DataReader* self, void* context);

// Entity naming Extensions ------------------------------------------------------------------------
/**
 * Name the domain participant
 *
 * @param self domain participant
 * @param name name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_name(dds_DomainParticipant* self, char* name/*[256]*/);

/**
 * Get domain participant's name
 *
 * @param self domain participant
 * @param name buffer to store name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_set_name(dds_DomainParticipant* self, char* name/*[256]*/);

/**
 * Name the publisher
 *
 * @param self publisher
 * @param name name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_get_name(dds_Publisher* self, char* name/*[256]*/);

/**
 * Get publisher's name
 *
 * @param self publisher
 * @param name buffer to store name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_set_name(dds_Publisher* self, char* name/*[256]*/);

/**
 * Name the datawriter
 *
 * @param self datawriter
 * @param name name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_name(dds_DataWriter* self, char* name/*[256]*/);

/**
 * Get datawriter's name
 *
 * @param self datawriter
 * @param name buffer to store name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_set_name(dds_DataWriter* self, char* name/*[256]*/);

/**
 * Name the subscriber
 *
 * @param self subscriber
 * @param name name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_get_name(dds_Subscriber* self, char* name/*[256]*/);

/**
 * Get subscriber's name
 *
 * @param self subscriber
 * @param name buffer to store name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_set_name(dds_Subscriber* self, char* name/*[256]*/);

/**
 * Name the datareader
 *
 * @param self datareader
 * @param name name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_name(dds_DataReader* self, char* name/*[256]*/);

/**
 * Get datareader's name
 *
 * @param self datareader
 * @param name buffer to store name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_set_name(dds_DataReader* self, char* name/*[256]*/);

/**
 * Creates a new rawly TypeSupport object
 *
 * @return newly created TypeSupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_TypeSupport_raw_create();

/**
 * Creates a new rawly TypeSupport object with key
 *
 * @return newly created TypeSupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_TypeSupport_raw_create_w_key();

/**
 * Destroy a TypeSupport object
 */
GURUMDDS_C_DLLAPI void dds_TypeSupport_destroy(dds_TypeSupport* self);

/**
 * Name the typesupport
 *
 * @param self typesupport
 * @param name name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_TypeSupport_get_name(dds_TypeSupport* self, char* name/*[256]*/);

/**
 * Get typesupport's name
 *
 * @param self typesupport
 * @param name buffer to store name
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_TypeSupport_set_name(dds_TypeSupport* self, char* name/*[256]*/);

/**
 * Get DynamicType
 *
 * @param self typesupport
 *
 * @return DynamicType object
 */
GURUMDDS_C_DLLAPI dds_DynamicType* dds_DynamicTypeSupport_get_dynamictype(dds_DynamicTypeSupport* self);

// Entity accessing extensions ------------------------------------------------------------------------
/**
 * Enumerates all domain participant that the domain participant factory has
 *
 * @param self domain participant factory
 * @param participants sequence of domain participant
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantFactory_get_contained_entities(dds_DomainParticipantFactory* self, dds_InstanceHandleSeq* participants);

/**
 * Enumerates all publisher, subscriber, topic and typesupport that the domain participant has (including builtin entity)
 *
 * @param self domain participant
 * @param publishers sequence of publishers
 * @param subscribers sequence of subscribers
 * @param topics sequence of topics
 * @param typesupports sequence of typesupports
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_contained_entities(dds_DomainParticipant* self, dds_InstanceHandleSeq* publishers, dds_InstanceHandleSeq* subscribers, dds_InstanceHandleSeq* topics, dds_InstanceHandleSeq* typesupports);

/**
 * Enumerates all datawriter that the publisher has (including builtin entity)
 *
 * @param self publisher
 * @param participants sequence of datawriter
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_get_contained_entities(dds_Publisher* self, dds_InstanceHandleSeq* datawriters);

/**
 * Enumerates all domain participant that the domain participant factory has (including builtin entity)
 *
 * @param self subscriber
 * @param participants sequence of datareader
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_get_contained_entities(dds_Subscriber* self, dds_InstanceHandleSeq* datareaders);

/**
 * Returns the domain participant with the given name
 *
 * @param self domain participant factory
 * @param name name of domain participant
 *
 * @return domain participant
 */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_DomainParticipantFactory_get_contained_participant_by_name(dds_DomainParticipantFactory* self, char* name);

/**
 * Returns the publisher with the given name
 *
 * @param self domain participant
 * @param name name of publisher
 *
 * @return publisher
 */
GURUMDDS_C_DLLAPI dds_Publisher* dds_DomainParticipant_get_contained_publisher_by_name(dds_DomainParticipant* self, char* name);

/**
 * Returns the subscriber with the given name
 *
 * @param self domain participant
 * @param name name of subscriber
 *
 * @return subscriber
 */
GURUMDDS_C_DLLAPI dds_Subscriber* dds_DomainParticipant_get_contained_subscriber_by_name(dds_DomainParticipant* self, char* name);

/**
 * Returns the topic with the given name
 *
 * @param self domain participant
 * @param name name of topic
 *
 * @return topic
 */
GURUMDDS_C_DLLAPI dds_Topic* dds_DomainParticipant_get_contained_topic_by_name(dds_DomainParticipant* self, char* name);

/**
 * Returns the typesupport with the given name
 *
 * @param self domain participant
 * @param name name of typesupport
 *
 * @return typesupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_DomainParticipant_get_contained_typesupport_by_name(dds_DomainParticipant* self, char* name);

/**
 * Returns the typesupport with the given type name
 *
 * @param self domain participant
 * @param type_name type name of typesupport
 *
 * @return typesupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_DomainParticipant_get_contained_typesupport_by_type_name(dds_DomainParticipant* self, char* type_name);

/**
 * Returns the datawriter with the given name
 *
 * @param self publisher
 * @param name name of datawriter
 *
 * @return datawriter
 */
GURUMDDS_C_DLLAPI dds_DataWriter* dds_Publisher_get_contained_datawriter_by_name(dds_Publisher* self, char* name);

/**
 * Returns the datareader with the given name
 *
 * @param self subscriber
 * @param name name of datareader
 *
 * @return datareader
 */
GURUMDDS_C_DLLAPI dds_DataReader* dds_Subscriber_get_contained_datareader_by_name(dds_Subscriber* self, char* name);

/**
 * Returns the typesupport of the datawriter
 *
 * @param self datawriter
 *
 * @return typesupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_DataWriter_get_typesupport(dds_DataWriter* self);

/**
 * Returns the typesupport of the datareader
 *
 * @param self datareader
 *
 * @return typesupport
 */
GURUMDDS_C_DLLAPI dds_TypeSupport* dds_DataReader_get_typesupport(dds_DataReader* self);

// Time extensions ------------------------------------------------------------------------
/**
 * Get current wall-clock time
 *
 * @param current_time wall-clock time
 */
GURUMDDS_C_DLLAPI void dds_Time_get_current_time(dds_Time_t* current_time);

/**
 * Sleep the current thread by the given amount of time
 *
 * @param sleep_time sleep time
 */
GURUMDDS_C_DLLAPI void dds_Time_sleep(dds_Time_t* sleep_time);

/**
 * Gets the time difference of two time objects
 *
 * @param lhs time object (left hand side)
 * @param rhs time object (right hand side)
 * @param diff time difference
 */
GURUMDDS_C_DLLAPI void dds_Time_diff(dds_Time_t* lhs, dds_Time_t* rhs, dds_Time_t* diff);

// Raw-Data extensions -----------------------------------------------------------------------------
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_raw_write(dds_DataWriter* self, const void* raw_data, uint32_t raw_data_size);
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_raw_write_w_timestamp(dds_DataWriter* self, const void* raw_data, uint32_t raw_data_size, const dds_Time_t* source_timestamp);
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_raw_write_w_sampleinfoex(dds_DataWriter* self, const void* raw_data, uint32_t raw_data_size, const dds_SampleInfoEx* sampleinfoex);

GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_raw_read(dds_DataReader* self, dds_InstanceHandle_t handle, dds_DataSeq* raw_data_values, dds_SampleInfoSeq* sample_infos, dds_UnsignedLongSeq* raw_data_sizes, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_raw_read_w_sampleinfoex(dds_DataReader* self, dds_InstanceHandle_t handle, dds_DataSeq* raw_data_values, dds_SampleInfoSeq* sample_infoexs, dds_UnsignedLongSeq* raw_data_sizes, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_raw_take(dds_DataReader* self, dds_InstanceHandle_t handle, dds_DataSeq* raw_data_values, dds_SampleInfoSeq* sample_infos, dds_UnsignedLongSeq* raw_data_sizes, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_raw_take_w_sampleinfoex(dds_DataReader* self, dds_InstanceHandle_t handle, dds_DataSeq* raw_data_values, dds_SampleInfoSeq* sample_infoexs, dds_UnsignedLongSeq* raw_data_sizes, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_raw_return_loan(dds_DataReader* self, dds_DataSeq* raw_data_values, dds_SampleInfoSeq* sample_infos, dds_UnsignedLongSeq* raw_data_sizes);

GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_guid(dds_DataWriter* self, uint8_t* guid);
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_guid_from_publication_handle(dds_DataReader* self, dds_InstanceHandle_t publication_handle, uint8_t* guid);

GURUMDDS_C_DLLAPI dds_String dds_ReturnCode_to_string(dds_ReturnCode_t retcode);

GURUMDDS_C_DLLAPI void* dds_malloc(size_t n);
GURUMDDS_C_DLLAPI void* dds_calloc(size_t nmemb, size_t s);
GURUMDDS_C_DLLAPI char* dds_strdup(const char* s);
GURUMDDS_C_DLLAPI uint16_t* dds_wcsdup(const uint16_t* s);
GURUMDDS_C_DLLAPI void dds_free(void* ptr);
GURUMDDS_C_DLLAPI dds_Wstring dds_wide_to_utf16(const wchar_t* s);
GURUMDDS_C_DLLAPI wchar_t* dds_utf16_to_wide(const dds_Wchar* s);

/**
 * @internal
 * Dump all of the contained policy by QoS to newly allocated string
 *
 * @param self QoS object
 *
 * @return newly allocated string
 */
GURUMDDS_C_DLLAPI char* dds_DomainParticipantQos_dump_as_string(dds_DomainParticipantQos* qos);

/** @see dds_DomainParticipantQos_dump_as_string */
GURUMDDS_C_DLLAPI char* dds_TopicQos_dump_as_string(dds_TopicQos* qos);

/** @see dds_DomainParticipantQos_dump_as_string */
GURUMDDS_C_DLLAPI char* dds_PublisherQos_dump_as_string(dds_PublisherQos* qos);

/** @see dds_DomainParticipantQos_dump_as_string */
GURUMDDS_C_DLLAPI char* dds_SubscriberQos_dump_as_string(dds_SubscriberQos* qos);

/** @see dds_DomainParticipantQos_dump_as_string */
GURUMDDS_C_DLLAPI char* dds_DataWriterQos_dump_as_string(dds_DataWriterQos* qos);

/** @see dds_DomainParticipantQos_dump_as_string */
GURUMDDS_C_DLLAPI char* dds_DataReaderQos_dump_as_string(dds_DataReaderQos* qos);



/**
 * Get TypeDeliveryService object from domain participant object
 * 
 * @param participant domain participant object
 * 
 * @return TypeDeliveryService object belong to domain participant
 **/
GURUMDDS_C_DLLAPI dds_TypeDeliveryService* dds_DomainParticipant_get_type_delivery_service(dds_DomainParticipant* self);

/**
 * Find TypeSupports by type_name. you must deallocate dds_DataSeq object
 * 
 * @param service TypeDeliveryService object
 * 
 * @return dds_DataSeq object has dds_TypeSupport objects
 **/
GURUMDDS_C_DLLAPI dds_DataSeq* dds_TypeDeliveryService_lookup(dds_TypeDeliveryService* service, const char* type_name);

/**
 * Get all type names from TypeDeliveryService object. you must deallocate dds_StringSeq object
 * 
 * @param service TypeDeliveryService object
 * 
 * @return dds_StringSeq has type names
 **/
GURUMDDS_C_DLLAPI dds_StringSeq* dds_TypeDeliveryService_get_type_names(dds_TypeDeliveryService* service);

#ifdef __cplusplus
}
#endif

#endif /* __GURUMDDS_DCPSX__ */
