#ifndef __GURUMDDS_DCPS__
#define __GURUMDDS_DCPS__

#if defined(_WIN64)
#if defined(GURUMDDS_C_DLLEXPORT)
#define GURUMDDS_C_DLLAPI __declspec(dllexport)
#else
#define GURUMDDS_C_DLLAPI __declspec(dllimport)
#endif
#else
#define GURUMDDS_C_DLLAPI
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file
 * DDS-DCPS Standard APIs
 *
 * This amalgamated header provides DCPS APIs for following modules
 * * Infrastructure Module
 * * Topic-Definition Module
 * * Publication Module
 * * Subscription Module
 */

#include <stddef.h>
#include <inttypes.h>
#include <stdbool.h>
#include <stdlib.h>

typedef char							dds_Char;
typedef char*							dds_String;
typedef uint16_t						dds_Wchar;
typedef uint16_t*						dds_Wstring;
typedef uint8_t							dds_Octet;
typedef int16_t							dds_Short;
typedef uint16_t						dds_UnsignedShort;
typedef int32_t							dds_Long;
typedef uint32_t						dds_UnsignedLong;
typedef int64_t							dds_LongLong;
typedef uint64_t						dds_UnsignedLongLong;
typedef float							dds_Float;
typedef double							dds_Double;
typedef uint8_t							dds_Boolean;
typedef struct dds_CharSeq				dds_CharSeq;
typedef struct dds_StringSeq			dds_StringSeq;
typedef struct dds_WcharSeq				dds_WcharSeq;
typedef struct dds_WstringSeq			dds_WstringSeq;
typedef struct dds_OctetSeq				dds_OctetSeq;
typedef struct dds_ShortSeq				dds_ShortSeq;
typedef struct dds_UnsignedShortSeq		dds_UnsignedShortSeq;
typedef struct dds_LongSeq				dds_LongSeq;
typedef struct dds_UnsignedLongSeq		dds_UnsignedLongSeq;
typedef struct dds_LongLongSeq			dds_LongLongSeq;
typedef struct dds_UnsignedLongLongSeq	dds_UnsignedLongLongSeq;
typedef struct dds_FloatSeq				dds_FloatSeq;
typedef struct dds_DoubleSeq			dds_DoubleSeq;
typedef struct dds_BooleanSeq			dds_BooleanSeq;
typedef struct dds_InstanceHandleSeq	dds_InstanceHandleSeq;
typedef struct dds_QosPolicyCountSeq 	dds_QosPolicyCountSeq;
typedef struct dds_DataReaderSeq		dds_DataReaderSeq;
typedef struct dds_DataWriterSeq		dds_DataWriterSeq;
typedef struct dds_ConditionSeq			dds_ConditionSeq;
typedef struct dds_SampleInfoSeq		dds_SampleInfoSeq;
typedef struct dds_DataSeq				dds_DataSeq;

typedef int32_t dds_DomainId_t;

typedef uintptr_t dds_InstanceHandle_t;

typedef struct {
	int32_t value[3];
} dds_BuiltinTopicKey_t;

typedef int32_t dds_ReturnCode_t;

typedef int32_t dds_QosPolicyId_t;

typedef struct {
	int32_t sec;
	uint32_t nanosec;
} dds_Duration_t;

typedef struct {
	int32_t sec;
	uint32_t nanosec;
} dds_Time_t;

// ----------------------------------------------------------------------
// Pre-defined values
// ----------------------------------------------------------------------
GURUMDDS_C_DLLAPI extern const dds_InstanceHandle_t dds_HANDLE_NIL;		//  0
GURUMDDS_C_DLLAPI extern const int32_t dds_LENGTH_UNLIMITED;			//  -1
GURUMDDS_C_DLLAPI extern const int32_t dds_DURATION_INFINITE_SEC;		//  0x7fffffff
GURUMDDS_C_DLLAPI extern const uint32_t dds_DURATION_INFINITE_NSEC;		//  0xffffffff
GURUMDDS_C_DLLAPI extern const int32_t dds_DURATION_ZERO_SEC;			//  0
GURUMDDS_C_DLLAPI extern const uint32_t dds_DURATION_ZERO_NSEC;			//  0
GURUMDDS_C_DLLAPI extern const int32_t dds_TIME_INVALID_SEC;			//  -1
GURUMDDS_C_DLLAPI extern const uint32_t dds_TIME_INVALID_NSEC;			//  0xffffffff

// ----------------------------------------------------------------------
// Return codes
// ----------------------------------------------------------------------
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_OK;						// 0
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_ERROR;					// 1
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_UNSUPPORTED;			// 2
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_BAD_PARAMETER;			// 3
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_PRECONDITION_NOT_MET;	// 4
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_OUT_OF_RESOURCES;		// 5
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_NOT_ENABLED;			// 6
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_IMMUTABLE_POLICY;		// 7
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_INCONSISTENT_POLICY;	// 8
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_ALREADY_DELETED;		// 9
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_TIMEOUT;				// 10
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_NO_DATA;				// 11
GURUMDDS_C_DLLAPI extern const dds_ReturnCode_t dds_RETCODE_ILLEGAL_OPERATION;		// 12

// ----------------------------------------------------------------------
// Status to support listeners and conditions
// ----------------------------------------------------------------------
typedef uint32_t dds_StatusKind;
typedef uint32_t dds_StatusMask; // bit-mask StatusKind
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_INCONSISTENT_TOPIC_STATUS;			// 0x0001 << 0
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_OFFERED_DEADLINE_MISSED_STATUS;		// 0x0001 << 1
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_REQUESTED_DEADLINE_MISSED_STATUS;		// 0x0001 << 2
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_OFFERED_INCOMPATIBLE_QOS_STATUS;		// 0x0001 << 5
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_REQUESTED_INCOMPATIBLE_QOS_STATUS;	// 0x0001 << 6
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_SAMPLE_LOST_STATUS;					// 0x0001 << 7
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_SAMPLE_REJECTED_STATUS;				// 0x0001 << 8
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_DATA_ON_READERS_STATUS;				// 0x0001 << 9
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_DATA_AVAILABLE_STATUS;				// 0x0001 << 10
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_LIVELINESS_LOST_STATUS;				// 0x0001 << 11
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_LIVELINESS_CHANGED_STATUS;			// 0x0001 << 12
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_PUBLICATION_MATCHED_STATUS;			// 0x0001 << 13
GURUMDDS_C_DLLAPI extern const dds_StatusKind dds_SUBSCRIPTION_MATCHED_STATUS;			// 0x0001 << 14

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
} dds_InconsistentTopicStatus;

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
} dds_SampleLostStatus;

typedef enum {
	dds_NOT_REJECTED,
	dds_REJECTED_BY_INSTANCES_LIMIT,
	dds_REJECTED_BY_SAMPLES_LIMIT,
	dds_REJECTED_BY_SAMPLES_PER_INSTANCE_LIMIT
} dds_SampleRejectedStatusKind;

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
	dds_SampleRejectedStatusKind last_reason;
	dds_InstanceHandle_t last_instance_handle;
} dds_SampleRejectedStatus;

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
} dds_LivelinessLostStatus;

typedef struct {
	int32_t alive_count;
	int32_t not_alive_count;
	int32_t alive_count_change;
	int32_t not_alive_count_change;
	dds_InstanceHandle_t last_publication_handle;
} dds_LivelinessChangedStatus;

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
	dds_InstanceHandle_t last_instance_handle;
} dds_OfferedDeadlineMissedStatus;

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
	dds_InstanceHandle_t last_instance_handle;
} dds_RequestedDeadlineMissedStatus;

typedef struct {
	dds_QosPolicyId_t policy_id;
	int32_t count;
} dds_QosPolicyCount;

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
	dds_QosPolicyId_t last_policy_id;
	dds_QosPolicyCount policies[25];
} dds_OfferedIncompatibleQosStatus;

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
	dds_QosPolicyId_t last_policy_id;
	dds_QosPolicyCount policies[25];
} dds_RequestedIncompatibleQosStatus;

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
	int32_t current_count;
	int32_t current_count_change;
	dds_InstanceHandle_t last_subscription_handle;
} dds_PublicationMatchedStatus;

typedef struct {
	int32_t total_count;
	int32_t total_count_change;
	int32_t current_count;
	int32_t current_count_change;
	dds_InstanceHandle_t last_publication_handle;
} dds_SubscriptionMatchedStatus;

typedef struct _dds_Listener					dds_Listener;
typedef struct _dds_TopicListener				dds_TopicListener;
typedef struct _dds_DataWriterListener			dds_DataWriterListener;
typedef struct _dds_DataReaderListener			dds_DataReaderListener;
typedef struct _dds_PublisherListener			dds_PublisherListener;
typedef struct _dds_SubscriberListener			dds_SubscriberListener;
typedef struct _dds_DomainParticipantListener	dds_DomainParticipantListener;
typedef struct _dds_Entity						dds_Entity;

/** TopicDescription represents the fact that both publications and subscriptions are tied to a single data-type */
typedef void									dds_TopicDescription;

/** Topic is the most basic description of the data to be published and subscribed */
typedef struct _dds_Topic						dds_Topic;

/** ContentFilteredTopic is a specialization of TopicDescription that allows for content-based subscriptions */
typedef struct _dds_ContentFilteredTopic		dds_ContentFilteredTopic;

/** MultiTopic is a specialization of TopicDescription that allows subscriptions to combine/filter/rearrange data coming from several topics */
typedef struct _dds_MultiTopic					dds_MultiTopic;

/** Publisher is the object responsible for the actual dissemination of publications */
typedef struct _dds_Publisher					dds_Publisher;

/** DataWriter allows the application to set the value of the data to be published under a given Topic */
typedef struct _dds_DataWriter					dds_DataWriter;

/** A Subscriber is the object responsible for the actual reception of the data resulting from its subscriptions */
typedef struct _dds_Subscriber					dds_Subscriber;

/** A DataReader allows the application to make a subscription */
typedef struct _dds_DataReader					dds_DataReader;

typedef void (*dds_TopicListener_on_inconsistent_topic)(const dds_Topic* the_topic, const dds_InconsistentTopicStatus* status);
struct _dds_TopicListener /* : Listener */ {
	dds_TopicListener_on_inconsistent_topic on_inconsistent_topic;
};

typedef void (*dds_DataWriterListener_on_offered_deadline_missed)(const dds_DataWriter* writer, const dds_OfferedDeadlineMissedStatus* status);
typedef void (*dds_DataWriterListener_on_offered_incompatible_qos)(const dds_DataWriter* writer, const dds_OfferedIncompatibleQosStatus* status);
typedef void (*dds_DataWriterListener_on_liveliness_lost)(const dds_DataWriter* writer, const dds_LivelinessLostStatus* status);
typedef void (*dds_DataWriterListener_on_publication_matched)(const dds_DataWriter* writer, const dds_PublicationMatchedStatus* status);
struct _dds_DataWriterListener /* : Listener */ {
	dds_DataWriterListener_on_offered_deadline_missed on_offered_deadline_missed;
	dds_DataWriterListener_on_offered_incompatible_qos on_offered_incompatible_qos;
	dds_DataWriterListener_on_liveliness_lost on_liveliness_lost;
	dds_DataWriterListener_on_publication_matched on_publication_matched;
};

struct _dds_PublisherListener /* : DataWriterListener */ {
	// DataWriterListener
	dds_DataWriterListener_on_offered_deadline_missed on_offered_deadline_missed;
	dds_DataWriterListener_on_offered_incompatible_qos on_offered_incompatible_qos;
	dds_DataWriterListener_on_liveliness_lost on_liveliness_lost;
	dds_DataWriterListener_on_publication_matched on_publication_matched;
};

typedef void (*dds_DataReaderListener_on_requested_deadline_missed)(const dds_DataReader* the_reader, const dds_RequestedDeadlineMissedStatus* status);
typedef void (*dds_DataReaderListener_on_requested_incompatible_qos)(const dds_DataReader* the_reader, const dds_RequestedIncompatibleQosStatus* status);
typedef void (*dds_DataReaderListener_on_sample_rejected)(const dds_DataReader* the_reader, const dds_SampleRejectedStatus* status);
typedef void (*dds_DataReaderListener_on_liveliness_changed)(const dds_DataReader* the_reader, const dds_LivelinessChangedStatus* status);
typedef void (*dds_DataReaderListener_on_data_available)(const dds_DataReader* the_reader);
typedef void (*dds_DataReaderListener_on_subscription_matched)(const dds_DataReader* the_reader, const dds_SubscriptionMatchedStatus* status);
typedef void (*dds_DataReaderListener_on_sample_lost)(const dds_DataReader* the_reader, const dds_SampleLostStatus* status);
struct _dds_DataReaderListener /* : Listener */ {
	dds_DataReaderListener_on_requested_deadline_missed on_requested_deadline_missed;
	dds_DataReaderListener_on_requested_incompatible_qos on_requested_incompatible_qos;
	dds_DataReaderListener_on_sample_rejected on_sample_rejected;
	dds_DataReaderListener_on_liveliness_changed on_liveliness_changed;
	dds_DataReaderListener_on_data_available on_data_available;
	dds_DataReaderListener_on_subscription_matched on_subscription_matched;
	dds_DataReaderListener_on_sample_lost on_sample_lost;
};

typedef void (*dds_SubscriberListener_on_data_on_readers)(const dds_Subscriber* the_subscriber);
struct _dds_SubscriberListener /* : DataReaderListener */ {
	// DataReaderListener
	dds_DataReaderListener_on_requested_deadline_missed on_requested_deadline_missed;
	dds_DataReaderListener_on_requested_incompatible_qos on_requested_incompatible_qos;
	dds_DataReaderListener_on_sample_rejected on_sample_rejected;
	dds_DataReaderListener_on_liveliness_changed on_liveliness_changed;
	dds_DataReaderListener_on_data_available on_data_available;
	dds_DataReaderListener_on_subscription_matched on_subscription_matched;
	dds_DataReaderListener_on_sample_lost on_sample_lost;

	// SubscriberListener
	dds_SubscriberListener_on_data_on_readers on_data_on_readers;
};

struct _dds_DomainParticipantListener /* : TopicListener, PublisherListener, SubscriberListener */ {
	// TopicListener : Listener
	dds_TopicListener_on_inconsistent_topic on_inconsistent_topic;

	// PublisherListener : DataWriterListener
	dds_DataWriterListener_on_offered_deadline_missed on_offered_deadline_missed;
	dds_DataWriterListener_on_offered_incompatible_qos on_offered_incompatible_qos;
	dds_DataWriterListener_on_liveliness_lost on_liveliness_lost;
	dds_DataWriterListener_on_publication_matched on_publication_matched;

	// SubscriberListener : DataReaderListener
	dds_DataReaderListener_on_requested_deadline_missed on_requested_deadline_missed;
	dds_DataReaderListener_on_requested_incompatible_qos on_requested_incompatible_qos;
	dds_DataReaderListener_on_sample_rejected on_sample_rejected;
	dds_DataReaderListener_on_liveliness_changed on_liveliness_changed;
	dds_DataReaderListener_on_data_available on_data_available;
	dds_DataReaderListener_on_subscription_matched on_subscription_matched;
	dds_DataReaderListener_on_sample_lost on_sample_lost;
	dds_SubscriberListener_on_data_on_readers on_data_on_readers;
};

// ----------------------------------------------------------------------
// WaitSet and Conditions
// ----------------------------------------------------------------------
/** A WaitSet object allows an application to wait until one or more of the attached Condition objects has a trigger_value ofTRUE or else until the timeout expires */
typedef struct _dds_WaitSet			dds_WaitSet;
/** A Condition is a root class for all the conditions that may be attached to a WaitSet */
typedef struct _dds_Condition		dds_Condition;
/** A GuardCondition object is a specific Condition whose trigger_value is completely under the control of the application */
typedef struct _dds_GuardCondition	dds_GuardCondition;
/** A StatusCondition object is a specific Condition that is associated with each Entity */
typedef struct _dds_StatusCondition	dds_StatusCondition;
/** ReadCondition objects are conditions specifically dedicated to read operations and attached to one DataReader */
typedef struct _dds_ReadCondition	dds_ReadCondition;
/** QueryCondition objects are specialized ReadCondition objects that allow the application to also specify a filter on the locally available data */
typedef struct _dds_QueryCondition	dds_QueryCondition;

// WaitSet
/**
 * Create a WaitSet
 *
 * @return newly created WaitSet
 */
GURUMDDS_C_DLLAPI dds_WaitSet* dds_WaitSet_create(void);

/**
 * Delete a WaitSet
 *
 * @param self waitset
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_WaitSet_delete(dds_WaitSet* self);

/**
 * Attaches a Condition to the WaitSet
 * It is possible to attach a Condition on a WaitSet that is currently being waited upon (via the wait operation)
 * In this case, if the Condition has a trigger_value of TRUE, then attaching the condition will unblock the WaitSet
 *
 * Adding a Condition that is already attached to the WaitSet has no effect
 *
 * @param self waitset
 * @param cond condition
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_WaitSet_attach_condition(dds_WaitSet* self, dds_Condition* cond);

/**
 * Detaches a Condition from the WaitSet
 * If the Condition was not attached to the Wait Set, the operation will return PRECONDITION_NOT_MET
 *
 * @param self waitset
 * @param cond condition
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_WaitSet_detach_condition(dds_WaitSet* self, dds_Condition* cond);

/**
 * Wait for the occurrence of certain conditions for calling thread
 * If none of the conditions attached to the WaitSet have a trigger_value of TRUE, the wait operation will block suspending the calling thread
 *
 * The result of the wait operation is the list of all the attached conditions that have a trigger_value of TRUE (i.e., the conditions that unblocked the wait)
 * The wait operation takes a timeout argument that specifies the maximum duration for the wait It this duration is exceeded and none of the attached Condition objects is true, wait will return with the return code TIMEOUT
 *
 * It is not allowed for more than one application thread to be waiting on the same WaitSet. If the wait operation is invoked on a WaitSet that already has a thread blocking on it, the operation will return immediately with the value PRECONDITION_NOT_ME
 *
 * @param self waitset
 * @param active_conditions activated conditions
 * @param timeout timeout
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_WaitSet_wait(dds_WaitSet* self, dds_ConditionSeq* active_conditions, dds_Duration_t* timeout);

/**
 * Retrieves the list of attached conditions
 *
 * @param self waitset
 * @param attached_conditions attached conditions
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_WaitSet_get_conditions(dds_WaitSet* self, dds_ConditionSeq* attached_conditions);

// Condition
/**
 * Retrieves the trigger_value of the Condition
 *
 * @param self condition
 *
 * @return trigger_value
 */
GURUMDDS_C_DLLAPI bool dds_Condition_get_trigger_value(dds_Condition* self);

// GuardCondition
/**
 * Create a GuardCondition
 *
 * @return newly created GuardCondition
 */
GURUMDDS_C_DLLAPI dds_GuardCondition* dds_GuardCondition_create(void);

/**
 * Delete a GuardCondition
 *
 * @param self guardcondition
 */
GURUMDDS_C_DLLAPI void dds_GuardCondition_delete(dds_GuardCondition* self);

/**
 * Retrieves the trigger_value of the GuardCondition
 *
 * @param self guardcondition
 *
 * @return trigger_value
 */
GURUMDDS_C_DLLAPI bool dds_GuardCondition_get_trigger_value(dds_GuardCondition* self);

/**
 * Sets the trigger_value of the GuardCondition
 *
 * @param self condition
 * @param value trigger_value
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_GuardCondition_set_trigger_value(dds_GuardCondition* self, bool value);

// StatusCondition
/**
 * Retrieves the trigger_value of the StatusCondition
 *
 * @param self statuscondition
 *
 * @return trigger_value
 */
GURUMDDS_C_DLLAPI bool dds_StatusCondition_get_trigger_value(dds_StatusCondition* self);

/**
 * Defines the list of communication statuses that are taken into account to determine the trigger_value of the StatusCondition
 * This operation may change the trigger_value of the StatusCondition
 *
 * @param self statuscondition
 * @param mask list of communication statuses
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_StatusCondition_set_enabled_statuses(dds_StatusCondition* self, dds_StatusMask mask);

/**
 * Retrieves the list of communication statuses that are taken into account to determine the trigger_value of the StatusCondition
 * This operation returns the statuses that were explicitly set on the last call to set_enabled_statuses or, if set_enabled_statuses was never called, the default list
 *
 * @param self statuscondition
 *
 * @return list of communication statuses
 */
GURUMDDS_C_DLLAPI dds_StatusMask dds_StatusCondition_get_enabled_statuses(dds_StatusCondition* self);


/**
 * Returns the Entity associated with the StatusCondition
 * Note that there is exactly one Entity associated with each StatusCondition
 *
 * @param self statuscondition
 *
 * @return Entity associated with the StatusCondition
 */
GURUMDDS_C_DLLAPI dds_Entity* dds_StatusCondition_get_entity(dds_StatusCondition* self);

// Sample states to support reads
typedef uint32_t dds_SampleStateKind;
GURUMDDS_C_DLLAPI extern const dds_SampleStateKind dds_READ_SAMPLE_STATE;		// 0x0001 << 0
GURUMDDS_C_DLLAPI extern const dds_SampleStateKind dds_NOT_READ_SAMPLE_STATE;	// 0x0001 << 1

// This is a bit-mask SampleStateKind
typedef uint32_t dds_SampleStateMask;
GURUMDDS_C_DLLAPI extern const dds_SampleStateMask dds_ANY_SAMPLE_STATE;		// 0xffff

// View states to support reads
typedef uint32_t dds_ViewStateKind;
GURUMDDS_C_DLLAPI extern const dds_ViewStateKind dds_NEW_VIEW_STATE;			// 0x0001 << 0
GURUMDDS_C_DLLAPI extern const dds_ViewStateKind dds_NOT_NEW_VIEW_STATE;		// 0x0001 << 1

// This is a bit-mask ViewStateKind
typedef uint32_t dds_ViewStateMask;
GURUMDDS_C_DLLAPI extern const dds_ViewStateMask dds_ANY_VIEW_STATE;			// 0xffff

// Instance states to support reads
typedef uint32_t dds_InstanceStateKind;
GURUMDDS_C_DLLAPI extern const dds_InstanceStateKind dds_ALIVE_INSTANCE_STATE;					// 0x0001 << 0
GURUMDDS_C_DLLAPI extern const dds_InstanceStateKind dds_NOT_ALIVE_DISPOSED_INSTANCE_STATE;		// 0x0001 << 1
GURUMDDS_C_DLLAPI extern const dds_InstanceStateKind dds_NOT_ALIVE_NO_WRITERS_INSTANCE_STATE;	// 0x0001 << 2

// This is a bit-mask InstanceStateKind
typedef uint32_t dds_InstanceStateMask;
GURUMDDS_C_DLLAPI extern const dds_InstanceStateMask dds_ANY_INSTANCE_STATE;		// 0xffff
GURUMDDS_C_DLLAPI extern const dds_InstanceStateMask dds_NOT_ALIVE_INSTANCE_STATE;	// 0x006

// ReadCondition
/**
 * Retrieves the trigger_value of the ReadCondition
 *
 * @param self readcondition
 *
 * @return trigger_value
 */
GURUMDDS_C_DLLAPI bool dds_ReadCondition_get_trigger_value(dds_ReadCondition* self);

/**
 * Returns the set of sample-states that are taken into account to determine the trigger_value of the ReadCondition
 * These are the sample-states specified when the ReadCondition was created
 *
 * @param self readcondition
 *
 * @return SampleStateMask
 */
GURUMDDS_C_DLLAPI dds_SampleStateMask dds_ReadCondition_get_sample_state_mask(dds_ReadCondition* self);

/**
 * Returns the set of view-states that are taken into account to determine the trigger_value of the ReadCondition
 * These are the view-states specified when the ReadCondition was created
 *
 * @param self readcondition
 *
 * @return ViewStateMask
 */
GURUMDDS_C_DLLAPI dds_ViewStateMask dds_ReadCondition_get_view_state_mask(dds_ReadCondition* self);

/**
 * Returns the set of instance-states that are taken into account to determine the trigger_value of the ReadCondition
 * These are the instance-states specified when the ReadCondition was created.
 *
 * @param self readcondition
 *
 * @return InstanceStateMask
 */
GURUMDDS_C_DLLAPI dds_InstanceStateMask dds_ReadCondition_get_instance_state_mask(dds_ReadCondition* self);

/**
 * Returns the DataReader associated with the ReadCondition
 * Note that there is exactly one DataReader associated with each ReadCondition
 *
 * @param self readcondition
 *
 * @return DataReader associated with the ReadCondition
 */
GURUMDDS_C_DLLAPI dds_DataReader* dds_ReadCondition_get_datareader(dds_ReadCondition* self);

// QueryCondition
/**
 * Retrieves the trigger_value of the QueryCondition
 *
 * @param self querycondition
 *
 * @return trigger_value
 */
GURUMDDS_C_DLLAPI bool dds_QueryCondition_get_trigger_value(dds_QueryCondition* self);

/**
 * Returns the set of sample-states that are taken into account to determine the trigger_value of the QueryCondition
 * These are the sample-states specified when the QueryCondition was created
 *
 * @param self querycondition
 *
 * @return SampleStateMask
 */
GURUMDDS_C_DLLAPI dds_SampleStateMask dds_QueryCondition_get_sample_state_mask(dds_QueryCondition* self);

/**
 * Returns the set of view-states that are taken into account to determine the trigger_value of the QueryCondition
 * These are the view-states specified when the QueryCondition was created
 *
 * @param self querycondition
 *
 * @return ViewStateMask
 */
GURUMDDS_C_DLLAPI dds_ViewStateMask dds_QueryCondition_get_view_state_mask(dds_QueryCondition* self);

/**
 * Returns the set of instance-states that are taken into account to determine the trigger_value of the QueryCondition
 * These are the instance-states specified when the QueryCondition was created.
 *
 * @param self querycondition
 *
 * @return InstanceStateMask
 */
GURUMDDS_C_DLLAPI dds_InstanceStateMask dds_QueryCondition_get_instance_state_mask(dds_QueryCondition* self);

/**
 * Returns the DataReader associated with the QueryCondition
 * Note that there is exactly one DataReader associated with each QueryCondition
 *
 * @param self querycondition
 *
 * @return DataReader associated with the QueryCondition
 */
GURUMDDS_C_DLLAPI dds_DataReader* dds_QueryCondition_get_datareader(dds_QueryCondition* self);

/**
 * Returns the query_expression associated with the QueryCondition
 * That is, the expression specified when the QueryCondition was created
 *
 * @param self querycondition
 *
 * @return query_expression
 */
GURUMDDS_C_DLLAPI const char* dds_QueryCondition_get_query_expression(dds_QueryCondition* self);

/**
 * Returns the query_parameters associated with the QueryCondition
 * That is, the parameters specified on the last successful call to set_query_parameters, or if set_query_parameters was never called, the arguments specified when the QueryCondition was created
 *
 * @param self querycondition
 * @param query_parameters query_parameters
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_QueryCondition_get_query_parameters(dds_QueryCondition* self, dds_StringSeq* query_parameters);

/**
 * Changes the query_parameters associated with the QueryCondition
 *
 * @param self querycondition
 * @param query_parameters query_parameters
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_QueryCondition_set_query_parameters(dds_QueryCondition* self, const dds_StringSeq* query_parameters);

// ----------------------------------------------------------------------
// Qos
// ----------------------------------------------------------------------
GURUMDDS_C_DLLAPI extern const char* dds_USERDATA_QOS_POLICY_NAME;						// "UserData"
GURUMDDS_C_DLLAPI extern const char* dds_DURABILITY_QOS_POLICY_NAME;					// "Durability"
GURUMDDS_C_DLLAPI extern const char* dds_PRESENTATION_QOS_POLICY_NAME;					// "Presentation"
GURUMDDS_C_DLLAPI extern const char* dds_DEADLINE_QOS_POLICY_NAME;						// "Deadline"
GURUMDDS_C_DLLAPI extern const char* dds_LATENCYBUDGET_QOS_POLICY_NAME;					// "LatencyBudget"
GURUMDDS_C_DLLAPI extern const char* dds_OWNERSHIP_QOS_POLICY_NAME;						// "Ownership"
GURUMDDS_C_DLLAPI extern const char* dds_OWNERSHIPSTRENGTH_QOS_POLICY_NAME;				// "OwnershipStrength"
GURUMDDS_C_DLLAPI extern const char* dds_LIVELINESS_QOS_POLICY_NAME;					// "Liveliness"
GURUMDDS_C_DLLAPI extern const char* dds_TIMEBASEDFILTER_QOS_POLICY_NAME;				// "TimeBasedFilter"
GURUMDDS_C_DLLAPI extern const char* dds_PARTITION_QOS_POLICY_NAME;						// "Partition"
GURUMDDS_C_DLLAPI extern const char* dds_RELIABILITY_QOS_POLICY_NAME;					// "Reliability"
GURUMDDS_C_DLLAPI extern const char* dds_DESTINATIONORDER_QOS_POLICY_NAME;				// "DestinationOrder"
GURUMDDS_C_DLLAPI extern const char* dds_HISTORY_QOS_POLICY_NAME;						// "History"
GURUMDDS_C_DLLAPI extern const char* dds_RESOURCELIMITS_QOS_POLICY_NAME;				// "ResourceLimits"
GURUMDDS_C_DLLAPI extern const char* dds_ENTITYFACTORY_QOS_POLICY_NAME;					// "EntityFactory"
GURUMDDS_C_DLLAPI extern const char* dds_WRITERDATALIFECYCLE_QOS_POLICY_NAME;			// "WriterDataLifecycle"
GURUMDDS_C_DLLAPI extern const char* dds_READERDATALIFECYCLE_QOS_POLICY_NAME;			// "ReaderDataLifecycle"
GURUMDDS_C_DLLAPI extern const char* dds_TOPICDATA_QOS_POLICY_NAME;						// "TopicData"
GURUMDDS_C_DLLAPI extern const char* dds_GROUPDATA_QOS_POLICY_NAME;						// "TransportPriority"
GURUMDDS_C_DLLAPI extern const char* dds_LIFESPAN_QOS_POLICY_NAME;						// "Lifespan"
GURUMDDS_C_DLLAPI extern const char* dds_DURABILITYSERVICE_QOS_POLICY_NAME;					// "DurabilityService"
GURUMDDS_C_DLLAPI extern const char* dds_DATAREPRESENTATION_QOS_POLICY_NAME;				// "DataRepresentation"
GURUMDDS_C_DLLAPI extern const char* dds_TYPECONSISTENCYENFORCEMENT_QOS_POLICY_NAME;	// "TypeConsistencyEnforcement"

GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_INVALID_QOS_POLICY_ID;						// 0
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_USERDATA_QOS_POLICY_ID;					// 1
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_DURABILITY_QOS_POLICY_ID;					// 2
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_PRESENTATION_QOS_POLICY_ID;				// 3
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_DEADLINE_QOS_POLICY_ID;					// 4
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_LATENCYBUDGET_QOS_POLICY_ID;				// 5
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_OWNERSHIP_QOS_POLICY_ID;					// 6
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_OWNERSHIPSTRENGTH_QOS_POLICY_ID;			// 7
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_LIVELINESS_QOS_POLICY_ID;					// 8
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_TIMEBASEDFILTER_QOS_POLICY_ID;				// 9
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_PARTITION_QOS_POLICY_ID;					// 10
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_RELIABILITY_QOS_POLICY_ID;					// 11
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_DESTINATIONORDER_QOS_POLICY_ID;			// 12
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_HISTORY_QOS_POLICY_ID;						// 13
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_RESOURCELIMITS_QOS_POLICY_ID;				// 14
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_ENTITYFACTORY_QOS_POLICY_ID;				// 15
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_WRITERDATALIFECYCLE_QOS_POLICY_ID;			// 16
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_READERDATALIFECYCLE_QOS_POLICY_ID;			// 17
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_TOPICDATA_QOS_POLICY_ID;					// 18
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_GROUPDATA_QOS_POLICY_ID;					// 19
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_TRANSPORTPRIORITY_QOS_POLICY_ID;			// 20
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_LIFESPAN_QOS_POLICY_ID;					// 21
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_DURABILITYSERVICE_QOS_POLICY_ID;			// 22
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_DATAREPRESENTATION_QOS_POLICY_ID;			// 23
GURUMDDS_C_DLLAPI extern const dds_QosPolicyId_t dds_TYPECONSISTENCYENFORCEMENT_QOS_POLICY_ID;	// 24

typedef int16_t dds_DataRepresentationId_t;
typedef struct dds_DataRepresentationIdSeq dds_DataRepresentationIdSeq;

GURUMDDS_C_DLLAPI extern const dds_DataRepresentationId_t dds_XCDR_DATA_REPRESENTATION;
GURUMDDS_C_DLLAPI extern const dds_DataRepresentationId_t dds_XML_DATA_REPRESENTATION;
GURUMDDS_C_DLLAPI extern const dds_DataRepresentationId_t dds_XCDR2_DATA_REPRESENTATION;

typedef struct {
	uint8_t		value[256];
	uint32_t	size;
} dds_UserDataQosPolicy;

typedef struct {
	uint8_t		value[256];
	uint32_t	size;
} dds_TopicDataQosPolicy;

typedef struct {
	uint8_t		value[256];
	uint32_t	size;
} dds_GroupDataQosPolicy;

typedef struct {
	int32_t value;
} dds_TransportPriorityQosPolicy;

typedef struct {
	dds_Duration_t duration;
} dds_LifespanQosPolicy;

typedef enum {
	dds_VOLATILE_DURABILITY_QOS,
	dds_TRANSIENT_LOCAL_DURABILITY_QOS,
	dds_TRANSIENT_DURABILITY_QOS,
	dds_PERSISTENT_DURABILITY_QOS
} dds_DurabilityQosPolicyKind;

typedef struct {
	dds_DurabilityQosPolicyKind kind;
} dds_DurabilityQosPolicy;

typedef enum {
	dds_INSTANCE_PRESENTATION_QOS,
	dds_TOPIC_PRESENTATION_QOS,
	dds_GROUP_PRESENTATION_QOS
} dds_PresentationQosPolicyAccessScopeKind;

typedef struct {
	dds_PresentationQosPolicyAccessScopeKind access_scope;
	bool coherent_access;
	bool ordered_access;
} dds_PresentationQosPolicy;

typedef struct {
	dds_Duration_t period;
} dds_DeadlineQosPolicy;

typedef struct {
	dds_Duration_t duration;
} dds_LatencyBudgetQosPolicy;

typedef enum {
	dds_SHARED_OWNERSHIP_QOS,
	dds_EXCLUSIVE_OWNERSHIP_QOS
} dds_OwnershipQosPolicyKind;

typedef struct {
	dds_OwnershipQosPolicyKind kind;
} dds_OwnershipQosPolicy;

typedef struct {
	int32_t value;
} dds_OwnershipStrengthQosPolicy;

typedef enum {
	dds_AUTOMATIC_LIVELINESS_QOS,
	dds_MANUAL_BY_PARTICIPANT_LIVELINESS_QOS,
	dds_MANUAL_BY_TOPIC_LIVELINESS_QOS
} dds_LivelinessQosPolicyKind;

typedef struct {
	dds_LivelinessQosPolicyKind kind;
	dds_Duration_t lease_duration;
} dds_LivelinessQosPolicy;

typedef struct {
	dds_Duration_t minimum_separation;
} dds_TimeBasedFilterQosPolicy;

typedef struct {
	dds_StringSeq* name;
} dds_PartitionQosPolicy;

typedef enum {
	dds_BEST_EFFORT_RELIABILITY_QOS = 1,
	dds_RELIABLE_RELIABILITY_QOS = 2
} dds_ReliabilityQosPolicyKind;

typedef struct {
	dds_ReliabilityQosPolicyKind kind;
	dds_Duration_t max_blocking_time;
} dds_ReliabilityQosPolicy;

typedef enum {
	dds_BY_RECEPTION_TIMESTAMP_DESTINATIONORDER_QOS,
	dds_BY_SOURCE_TIMESTAMP_DESTINATIONORDER_QOS
} dds_DestinationOrderQosPolicyKind;

typedef struct {
	dds_DestinationOrderQosPolicyKind kind;
} dds_DestinationOrderQosPolicy;

typedef enum {
	dds_KEEP_LAST_HISTORY_QOS,
	dds_KEEP_ALL_HISTORY_QOS
} dds_HistoryQosPolicyKind;

typedef struct {
	dds_HistoryQosPolicyKind kind;
	int32_t depth;
} dds_HistoryQosPolicy;

typedef struct {
	int32_t max_samples;
	int32_t max_instances;
	int32_t max_samples_per_instance;
} dds_ResourceLimitsQosPolicy;

typedef struct {
	bool autoenable_created_entities;
} dds_EntityFactoryQosPolicy;

typedef struct {
	bool autodispose_unregistered_instances;
} dds_WriterDataLifecycleQosPolicy;

typedef struct {
	dds_Duration_t autopurge_nowriter_samples_delay;
	dds_Duration_t autopurge_disposed_samples_delay;
} dds_ReaderDataLifecycleQosPolicy;

typedef struct {
	dds_Duration_t service_cleanup_delay;
	dds_HistoryQosPolicyKind history_kind;
	int32_t history_depth;
	int32_t max_samples;
	int32_t max_instances;
	int32_t max_samples_per_instance;
} dds_DurabilityServiceQosPolicy;

typedef struct {
	dds_DataRepresentationIdSeq* value;
} dds_DataRepresentationQosPolicy;

typedef enum {
	dds_DISALLOW_TYPE_COERCION_TYPECONSISTENCY_QOS,
	dds_ALLOW_TYPE_COERCION_TYPECONSISTENCY_QOS
} dds_TypeConsistencyKind;

typedef struct {
	dds_TypeConsistencyKind kind;
	bool ignore_sequence_bounds;
	bool ignore_string_bounds;
	bool ignore_member_names;
	bool prevent_type_widening;
	bool force_type_validation;
} dds_TypeConsistencyEnforcementQosPolicy;

typedef struct {
	dds_EntityFactoryQosPolicy entity_factory;
} dds_DomainParticipantFactoryQos;

typedef struct {
	dds_UserDataQosPolicy user_data;
	dds_EntityFactoryQosPolicy entity_factory;
} dds_DomainParticipantQos;

typedef struct {
	dds_TopicDataQosPolicy topic_data;
	dds_DurabilityQosPolicy durability;
	dds_DurabilityServiceQosPolicy durability_service;
	dds_DeadlineQosPolicy deadline;
	dds_LatencyBudgetQosPolicy latency_budget;
	dds_LivelinessQosPolicy liveliness;
	dds_ReliabilityQosPolicy reliability;
	dds_DestinationOrderQosPolicy destination_order;
	dds_HistoryQosPolicy history;
	dds_ResourceLimitsQosPolicy resource_limits;
	dds_TransportPriorityQosPolicy transport_priority;
	dds_LifespanQosPolicy lifespan;
	dds_OwnershipQosPolicy ownership;
	dds_DataRepresentationQosPolicy data_representation;
} dds_TopicQos;

typedef struct {
	dds_DurabilityQosPolicy durability;
	dds_DurabilityServiceQosPolicy durability_service;
	dds_DeadlineQosPolicy deadline;
	dds_LatencyBudgetQosPolicy latency_budget;
	dds_LivelinessQosPolicy liveliness;
	dds_ReliabilityQosPolicy reliability;
	dds_DestinationOrderQosPolicy destination_order;
	dds_HistoryQosPolicy history;
	dds_ResourceLimitsQosPolicy resource_limits;
	dds_TransportPriorityQosPolicy transport_priority;
	dds_LifespanQosPolicy lifespan;
	dds_UserDataQosPolicy user_data;
	dds_OwnershipQosPolicy ownership;
	dds_OwnershipStrengthQosPolicy ownership_strength;
	dds_WriterDataLifecycleQosPolicy writer_data_lifecycle;
	dds_DataRepresentationQosPolicy data_representation;
} dds_DataWriterQos;

typedef struct {
	dds_PresentationQosPolicy presentation;
	dds_PartitionQosPolicy partition;
	dds_GroupDataQosPolicy group_data;
	dds_EntityFactoryQosPolicy entity_factory;
} dds_PublisherQos;

typedef struct {
	dds_DurabilityQosPolicy durability;
	dds_DeadlineQosPolicy deadline;
	dds_LatencyBudgetQosPolicy latency_budget;
	dds_LivelinessQosPolicy liveliness;
	dds_ReliabilityQosPolicy reliability;
	dds_DestinationOrderQosPolicy destination_order;
	dds_HistoryQosPolicy history;
	dds_ResourceLimitsQosPolicy resource_limits;
	dds_UserDataQosPolicy user_data;
	dds_OwnershipQosPolicy ownership;
	dds_TimeBasedFilterQosPolicy time_based_filter;
	dds_ReaderDataLifecycleQosPolicy reader_data_lifecycle;
	dds_DataRepresentationQosPolicy data_representation;
	dds_TypeConsistencyEnforcementQosPolicy type_consistency;
} dds_DataReaderQos;

typedef struct {
	dds_PresentationQosPolicy presentation;
	dds_PartitionQosPolicy partition;
	dds_GroupDataQosPolicy group_data;
	dds_EntityFactoryQosPolicy entity_factory;
} dds_SubscriberQos;

/**
 * Copies the QoS policies of the DomainParticipantFactory
 *
 * @param dst destination domain participant factory qos
 * @param src source domain participant factory qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantFactoryQos_copy(dds_DomainParticipantFactoryQos* dst, const dds_DomainParticipantFactoryQos* src);

/**
 * Finalizes the QoS policies of the DomainParticipantFactory. This operation frees allocated memories for DomainParticipantFactoryQos.
 * Without PublisherQos and SubscriberQos, the finalize operation does nothing.
 *
 * @param qos domain participant factory qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantFactoryQos_finalize(dds_DomainParticipantFactoryQos* qos);

/**
 * Copies the QoS policies of the DomainParticipant
 *
 * @param dst destination domain participant qos
 * @param src source domain participant qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantQos_copy(dds_DomainParticipantQos* dst, const dds_DomainParticipantQos* src);

/**
 * Finalizes the QoS policies of the DomainParticipant. This operation frees allocated memories for DomainParticipantQos.
 * Without PublisherQos and SubscriberQos, the finalize operation does nothing.
 *
 * @param qos domain participant qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantQos_finalize(dds_DomainParticipantQos* qos);

/**
 * Copies the QoS policies of the Publisher
 *
 * @param dst destination publisher qos
 * @param src source publisher qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_PublisherQos_copy(dds_PublisherQos* dst, const dds_PublisherQos* src);

/**
 * Finalizes the QoS policies of the Publisher. This operation frees allocated memories for PublisherQos.
 *
 * @param qos publisher qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_PublisherQos_finalize(dds_PublisherQos* qos);

/**
 * Copies the QoS policies of the Subscriber
 *
 * @param dst destination subscriber qos
 * @param src source subscriber qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_SubscriberQos_copy(dds_SubscriberQos* dst, const dds_SubscriberQos* src);

/**
 * Finalizes the QoS policies of the Subscriber. This operation frees allocated memories for SubscriberQos.
 *
 * @param qos subscriber qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_SubscriberQos_finalize(dds_SubscriberQos* qos);

/**
 * Copies the QoS policies of the Topic
 *
 * @param dst destination topic qos
 * @param src source topic qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_TopicQos_copy(dds_TopicQos* dst, const dds_TopicQos* src);

/**
 * Finalizes the QoS policies of the Topic. This operation frees allocated memories for TopicQos.
 * Without PublisherQos and SubscriberQos, the finalize operation does nothing.
 *
 * @param qos topic qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_TopicQos_finalize(dds_TopicQos* qos);

/**
 * Copies the QoS policies of the DataWriter
 *
 * @param dst destination datawriter qos
 * @param src source datawriter qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriterQos_copy(dds_DataWriterQos* dst, const dds_DataWriterQos* src);

/**
 * Finalizes the QoS policies of the DataWriter. This operation frees allocated memories for DataWriterQos.
 * Without PublisherQos and SubscriberQos, the finalize operation does nothing.
 *
 * @param qos datawriter qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriterQos_finalize(dds_DataWriterQos* qos);

/**
 * Copies the QoS policies of the DataReader
 *
 * @param dst destination datareader qos
 * @param src source datareader qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReaderQos_copy(dds_DataReaderQos* dst, const dds_DataReaderQos* src);

/**
 * Finalizes the QoS policies of the DataReader. This operation frees allocated memories for DataReaderQos.
 * Without PublisherQos and SubscriberQos, the finalize operation does nothing.
 *
 * @param qos datareader qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReaderQos_finalize(dds_DataReaderQos* qos);

// ----------------------------------------------------------------------
typedef struct _dds_TypeInformation dds_TypeInformation;
typedef struct dds_TypeInformationSeq dds_TypeInformationSeq;

typedef struct {
	dds_BuiltinTopicKey_t key;
	dds_UserDataQosPolicy user_data;
} dds_ParticipantBuiltinTopicData;

typedef struct {
	dds_BuiltinTopicKey_t key;
	char name[256];
	char type_name[256];
	dds_TypeInformation* type_information;
	dds_DurabilityQosPolicy durability;
	dds_DurabilityServiceQosPolicy durability_service;
	dds_DeadlineQosPolicy deadline;
	dds_LatencyBudgetQosPolicy latency_budget;
	dds_LivelinessQosPolicy liveliness;
	dds_ReliabilityQosPolicy reliability;
	dds_TransportPriorityQosPolicy transport_priority;
	dds_LifespanQosPolicy lifespan;
	dds_DestinationOrderQosPolicy destination_order;
	dds_HistoryQosPolicy history;
	dds_ResourceLimitsQosPolicy resource_limits;
	dds_OwnershipQosPolicy ownership;
	dds_TopicDataQosPolicy topic_data;
	dds_DataRepresentationQosPolicy data_representation;
} dds_TopicBuiltinTopicData;

typedef struct {
	dds_BuiltinTopicKey_t key;
	dds_BuiltinTopicKey_t participant_key;
	char topic_name[256];
	char type_name[256];
	dds_TypeInformation* type_information;
	dds_DurabilityQosPolicy durability;
	dds_DurabilityServiceQosPolicy durability_service;
	dds_DeadlineQosPolicy deadline;
	dds_LatencyBudgetQosPolicy latency_budget;
	dds_LivelinessQosPolicy liveliness;
	dds_ReliabilityQosPolicy reliability;
	dds_LifespanQosPolicy lifespan;
	dds_UserDataQosPolicy user_data;
	dds_OwnershipQosPolicy ownership;
	dds_OwnershipStrengthQosPolicy ownership_strength;
	dds_DestinationOrderQosPolicy destination_order;
	dds_PresentationQosPolicy presentation;
	dds_PartitionQosPolicy partition;
	dds_TopicDataQosPolicy topic_data;
	dds_GroupDataQosPolicy group_data;
	dds_DataRepresentationQosPolicy data_representation;
} dds_PublicationBuiltinTopicData;

typedef struct {
	dds_BuiltinTopicKey_t key;
	dds_BuiltinTopicKey_t participant_key;
	char topic_name[256];
	char type_name[256];
	dds_TypeInformation* type_information;
	dds_DurabilityQosPolicy durability;
	dds_DeadlineQosPolicy deadline;
	dds_LatencyBudgetQosPolicy latency_budget;
	dds_LivelinessQosPolicy liveliness;
	dds_ReliabilityQosPolicy reliability;
	dds_OwnershipQosPolicy ownership;
	dds_DestinationOrderQosPolicy destination_order;
	dds_UserDataQosPolicy user_data;
	dds_TimeBasedFilterQosPolicy time_based_filter;
	dds_PresentationQosPolicy presentation;
	dds_PartitionQosPolicy partition;
	dds_TopicDataQosPolicy topic_data;
	dds_GroupDataQosPolicy group_data;
	dds_DataRepresentationQosPolicy data_representation;
	dds_TypeConsistencyEnforcementQosPolicy type_consistency;
} dds_SubscriptionBuiltinTopicData;

/**
 * Enables the Entity
 *
 * The enable operation is idempotent. Calling enable on an already enabled Entity returns OK and has no effect
 *
 * @param self entity
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Entity_enable(dds_Entity* self);

/**
 * Retrieves the StatusCondition associated with the Entity
 *
 * @param self entity
 *
 * @return statuscondition
 */
GURUMDDS_C_DLLAPI dds_StatusCondition* dds_Entity_get_statuscondition(dds_Entity* self);

/**
 * Retrieves the list of communication statuses in the Entity that are 'triggered'
 *
 * @param self entity
 *
 * @return statusmask
 */
GURUMDDS_C_DLLAPI dds_StatusMask dds_Entity_get_status_changes(dds_Entity* self);

/**
 * Retrieves the InstanceHandle_t that represents the Entity
 *
 * @param self entity
 *
 * @return instance handle
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_Entity_get_instance_handle(dds_Entity* self);

// ----------------------------------------------------------------------
/** DomainParticipant represents the participation of the application on a communication plane */
typedef struct _dds_DomainParticipant dds_DomainParticipant;

/**
 * Sets the QoS policies of the DomainParticipant
 *
 * @param self domain participant
 * @param qos domain participant qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_set_qos(dds_DomainParticipant* self, const dds_DomainParticipantQos* qos);

/**
 * Retrieves the QoS policies of the DomainParticipant
 *
 * @param self domain participant
 * @param qos domain participant qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_qos(dds_DomainParticipant* self, dds_DomainParticipantQos* qos);

/**
 * Installs a Listener on the DomainParticipant. The listener will only be invoked on the changes of communication status indicated by the specified mask
 *
 * Only one listener can be attached. If a listener was already set, the operation set_listener will replace it with the new one. Consequently if the value 'nil' is passed for the listener parameter to the set_listener operation, any existing listener will be removed
 *
 * @param self domain participant
 * @param l domain participant listener
 * @param mask domain participant listener mask
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_set_listener(dds_DomainParticipant* self, const dds_DomainParticipantListener* l, dds_StatusMask mask);

/**
 * Retrieves the existing Listener attached to the DomainParticipant
 *
 * @param self domain participant
 *
 * @return domain participant listener
 */
GURUMDDS_C_DLLAPI dds_DomainParticipantListener dds_DomainParticipant_get_listener(dds_DomainParticipant* self);

/**
 * Enables the DomainParticipant
 *
 * The enable operation is idempotent. Calling enable on an already enabled Entity returns OK and has no effect
 *
 * @param self domain participant
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_enable(dds_DomainParticipant* self);

/**
 * Retrieves the StatusCondition associated with the DomainParticipant
 *
 * @param self domain participant
 *
 * @return statuscondition
 */
GURUMDDS_C_DLLAPI dds_StatusCondition* dds_DomainParticipant_get_statuscondition(dds_DomainParticipant* self);

/**
 * Retrieves the list of communication statuses in the Entity that are 'triggered'
 *
 * @param self domain participant
 *
 * @return statusmask
 */
GURUMDDS_C_DLLAPI dds_StatusMask dds_DomainParticipant_get_status_changes(dds_DomainParticipant* self);

/**
 * Retrieves the InstanceHandle_t that represents the DomainParticipant
 *
 * @param self domain participant
 *
 * @return instance handle
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DomainParticipant_get_instance_handle(dds_DomainParticipant* self);

/**
 * Creates a Publisher with the desired QoS policies and attaches to it the specified PublisherListener. The created Publisher belongs to the DomainParticipant that is its factory
 *
 * If the specified QoS policies are not consistent, the operation will fail and no Publisher will be created. The special value PUBLISHER_QOS_DEFAULT can be used to indicate that the Publisher should be created with the default Publisher QoS set in the factory. The use of this value is equivalent to the application obtaining the default Publisher QoS by means of the operation get_default_publisher_qos and using the resulting QoS to create the Publisher
 *
 * @param self domain participant
 * @param qos publisher qos
 * @param a_listener publisher listener
 * @param mask publisher listener mask
 *
 * @return newly created publisher
 */
GURUMDDS_C_DLLAPI dds_Publisher* dds_DomainParticipant_create_publisher(dds_DomainParticipant* self, const dds_PublisherQos* qos, const dds_PublisherListener* a_listener, dds_StatusMask mask);

/**
 * Deletes an existing Publisher
 *
 * A Publisher cannot be deleted if it has any attached DataWriter objects. If delete_publisher is called on a Publisher with existing DataWriter object, it will return PRECONDITION_NOT_MET. The delete_publisher operation must be called on the same DomainParticipant object used to create the Publisher. If delete_publisher is called on a different DomainParticipant, the operation will have no effect and it will return PRECONDITION_NOT_MET
 *
 * @param self domain participant
 * @param publisher publisher
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_delete_publisher(dds_DomainParticipant* self, dds_Publisher* publisher);

/**
 * Creates a Subscriber with the desired QoS policies and attaches to it the specified SubscriberListener
 *
 * If the specified QoS policies are not consistent, the operation will fail and no Subscriber will be created. The special value SUBSCRIBER_QOS_DEFAULT can be used to indicate that the Subscriber should be created with the default Subscriber QoS set in the factory. The use of this value is equivalent to the application obtaining the default Subscriber QoS by means of the operation get_default_subscriber_qos and using the resulting QoS to create the Subscriber
 *
 * The created Subscriber belongs to the DomainParticipant that is its factory
 *
 * @param self domain participant
 * @param qos subscriber qos
 * @param a_listener subscriber listener
 * @param mask subscriber listener mask
 *
 * @return newly created subscriber
 */
GURUMDDS_C_DLLAPI dds_Subscriber* dds_DomainParticipant_create_subscriber(dds_DomainParticipant* self, const dds_SubscriberQos* qos, const dds_SubscriberListener* a_listener, dds_StatusMask mask);

/**
 * Deletes an existing Subscriber
 *
 * A Subscriber cannot be deleted if it has any attached DataReader objects. If the delete_subscriber operation is called on a Subscriber with existing DataReader objects, it will return PRECONDITION_NOT_MET.The delete_subscriber operation must be called on the same DomainParticipant object used to create the Subscriber. If delete_subscriber is called on a different DomainParticipant, the operation will have no effect and it will return PRECONDITION_NOT_MET
 *
 * @param self domain participant
 * @param subscriber subscriber
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_delete_subscriber(dds_DomainParticipant* self, dds_Subscriber* subscriber);

/**
 * Retrieves the built-in Subscriber
 *
 * Each DomainParticipant contains several built-in Topic objects as well as corresponding DataReader objects to access them. All these DataReader objects belong to a single built-in Subscriber. The built-in Topics are used to communicate information about other DomainParticipant, Topic, DataReader, and DataWriterobjects
 *
 * @param self domain participant
 *
 * @return built-in subscriber
 */
GURUMDDS_C_DLLAPI dds_Subscriber* dds_DomainParticipant_get_builtin_subscriber(dds_DomainParticipant* self);

/**
 * Creates a Topic with the desired QoS policies and attaches to it the specified TopicListener
 *
 * If the specified QoS policies are not consistent, the operation will fail and no Topic will be created.The special value TOPIC_QOS_DEFAULT can be used to indicate that the Topic should be created with the default Topic QoS set in the factory. The use of this value is equivalent to the application obtaining the default Topic QoS by means of the operation get_default_topic_qos and using the resulting QoS to create the Topic
 *
 * The created Topic belongs to the DomainParticipant that is its factory
 *
 * The Topic is bound to a type described by the type_name argument. Prior to creating a Topic the type must have been registered with the Service. This is done using the register_type operation on a derived class of the TypeSupport interface
 *
 * @param self domain participant
 * @param topic_name topic name
 * @param type_name type name
 * @param qos topic qos
 * @param a_listener topic listener
 * @param mask topic listener mask
 *
 * @return newly created topic
 */
GURUMDDS_C_DLLAPI dds_Topic* dds_DomainParticipant_create_topic(dds_DomainParticipant* self, const char* topic_name, const char* type_name, const dds_TopicQos* qos, const dds_TopicListener* a_listener, dds_StatusMask mask);

/**
 * Deletes a Topic
 *
 * The deletion of a Topic is not allowed if there are any existing DataReader, DataWriter, ContentFilteredTopic, or MultiTopicobjects that are using the Topic. If the delete_topic operation is called on a Topic with any of these existing objects attached to it, it will return PRECONDITION_NOT_MET. The delete_topic operation must be called on the same DomainParticipant object used to create the Topic. If delete_topic is called on a different DomainParticipant, the operation will have no effect and it will return PRECONDITION_NOT_MET
 *
 * @param self domain participant
 * @param a_topic topic
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_delete_topic(dds_DomainParticipant* self, dds_Topic* a_topic);

/**
 * Finds an existing (or ready to exist) enabled Topic, based on its name
 *
 * This operation takes as arguments the name of the Topic and a timeout. If a Topic of the same name already exists, it gives access to it, otherwise it waits (blocks the caller) until another mechanism creates it. If the operation times-out, a 'nil' value is returned
 *
 * @param self domain participant
 * @param topic_name topic name
 * @param timeout timeout
 *
 * @return topic
 */
GURUMDDS_C_DLLAPI dds_Topic* dds_DomainParticipant_find_topic(dds_DomainParticipant* self, const char* topic_name, const dds_Duration_t* timeout);

/**
 * Gives access to an existing locally-created TopicDescription, based on its name
 *
 * If the operation fails to locate a TopicDescription, a 'nil' value is returned
 *
 * @param self domain participant
 * @param name topic description name
 *
 * @return topic description
 */
GURUMDDS_C_DLLAPI dds_TopicDescription* dds_DomainParticipant_lookup_topicdescription(dds_DomainParticipant* self, const char* name);

/**
 * Gives access to an existing locally-created TopicDescription, based on its name and index.
 *
 * If the operation fails to locate a TopicDescription, a 'nil' value is returned
 *
 * @param self domain participant
 * @param name topic description name
 * @param index topic description index
 *
 * @return topic description
 */
GURUMDDS_C_DLLAPI dds_TopicDescription* dds_DomainParticipant_lookup_topicdescription_w_index(dds_DomainParticipant* self, const char* name, dds_UnsignedLong index);

/**
 * Creates a ContentFilteredTopic
 *
 * ContentFilteredTopic can be used to do content-based subscriptions. The related Topic being subscribed to is specified by means of the related_topic parameter. The ContentFilteredTopic only relates to samples published under that Topic,  filtered according to their content. The filtering is done by means of evaluating a logical expression that involves the values of some of the data-fields in the sample
 *
 * The logical expression is derived from the filter_expression and expression_parameters arguments. The syntax of the filter expression and parameters is described in DCPS Standard
 *
 * @param self domain participant
 * @param name topic name
 * @param related_topic related topic
 * @param filter_expression filter_expression
 * @param expression_parameters expression_parameters
 *
 * @return newly created ContentFilteredTopic
 */
GURUMDDS_C_DLLAPI dds_ContentFilteredTopic* dds_DomainParticipant_create_contentfilteredtopic(dds_DomainParticipant* self, const char* name, const dds_Topic* related_topic, const char* filter_expression, const dds_StringSeq* expression_parameters);

/**
 * Deletes a ContentFilteredTopic
 *
 * The deletion of a ContentFilteredTopic is not allowed if there are any existing DataReader objects that are using the ContentFilteredTopic. If the delete_contentfilteredtopic operation is called on a ContentFilteredTopic with existing DataReader objects attached to it, it will return PRECONDITION_NOT_MET.The delete_contentfilteredtopic operation must be called on the same DomainParticipant object used to create the ContentFilteredTopic. If delete_contentfilteredtopic is called on a different DomainParticipant, the operation will have no effect and it will return PRECONDITION_NOT_MET
 *
 * @param self domain participant
 * @param a_contentfilteredtopic contentfilteredtopic
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_delete_contentfilteredtopic(dds_DomainParticipant* self, dds_ContentFilteredTopic* a_contentfilteredtopic);

/**
 * Creates a MultiTopic
 *
 * MultiTopic can be used to subscribe to multiple topics and combine/filter the received data into a resulting type. In particular, MultiTopic provides a content-based subscription mechanism. The resulting type is specified by the type_name argument. Prior to creating a MultiTopic the type must have been registered with the Service. This is done using the register_type operation on a derived class of the TypeSupport
 *
 * The list of topics and the logic used to combine filter and re-arrange the information from each Topic are specified using the subscription_expression and expression_parameters arguments. The syntax of the expression and parameters is described in OMG DCPS Standard
 *
 * @param self domain participant
 * @param name topic name
 * @param type_name type name
 * @param subscription_expression subscription expression
 * @param expression_parameters expression parameters
 *
 * @return newly created MultiTopic
 */
GURUMDDS_C_DLLAPI dds_MultiTopic* dds_DomainParticipant_create_multitopic(dds_DomainParticipant* self, const char* name, const char* type_name, const char* subscription_expression, const dds_StringSeq* expression_parameters);

/**
 * Deletes a MultiTopic
 *
 * The deletion of a MultiTopic is not allowed if there are any existing DataReader objects that are using the MultiTopic. If the delete_multitopic operation is called on a MultiTopic with existing DataReader objects attached to it, it will return PRECONDITION_NOT_MET. The delete_multitopic operation must be called on the same DomainParticipant object used to create the MultiTopic. If delete_multitopic is called on a different DomainParticipant, the operation will have no effect and it will return PRECONDITION_NOT_MET
 *
 * @param self domain participant
 * @param a_multitopic multitopic
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_delete_multitopic(dds_DomainParticipant* self, dds_MultiTopic* a_multitopic);

/**
 * Deletes all the entities that were created by means of the “create” operations on the DomainParticipant
 *
 * Deletes all contained Publisher, Subscriber, Topic, ContentFilteredTopic, and MultiTopic. Prior to deleting each contained entity, this operation will recursively call the corresponding delete_contained_entitiesoperation on each contained entity (if applicable). This pattern is applied recursively. In this manner the operation delete_contained_entities on the DomainParticipant will end up deleting all the entities recursively contained in the DomainParticipant, that is also the DataWriter, DataReader, as well as the QueryCondition and ReadCondition objects belonging to the contained DataReaders.The operation will return PRECONDITION_NOT_MET if the any of the contained entities is in a state where it cannot be deleted
 *
 * @param self domain participant
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_delete_contained_entities(dds_DomainParticipant* self);

/**
 * Instruct the Service to locally ignore a remote domain participant. From that point onwards the Service will locally behave as if the remote participant did not exist. This means it will ignore any Topic, publication, or subscription that originates on that domain participant
 *
 * This operation is not reversible
 *
 * @param self domain participant
 * @param handle remote domain participant handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_ignore_participant(dds_DomainParticipant* self, dds_InstanceHandle_t handle);

/**
 * Instruct the Service to locally ignore a Topic. This means it will locally ignore any publication or subscription to the Topic
 *
 * This operation is not reversible
 *
 * @param self domain participant
 * @param handle topic handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_ignore_topic(dds_DomainParticipant* self, dds_InstanceHandle_t handle);

/**
 * Instruct the Service to locally ignore a remote publication. After this call, any data written related to that publication will be ignored
 *
 * The DataWriter to ignore is identified by the handle argument. This handle is the one that appears in the SampleInfo retrieved when reading the data-samples from the built-in DataReader to the “DCPSPublication” topic.The ignore_publication operation is not required to be reversible. The Service offers no means to reverse it.Possible error codes returned in addition to the standar
 *
 * This operation is not reversible
 *
 * @param self domain participant
 * @param handle publication handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_ignore_publication(dds_DomainParticipant* self, dds_InstanceHandle_t handle);

/**
 * Instruct the Service to locally ignore a remote subscription. After this call, any data received related to that subscription will be ignored
 *
 * This operation is not reversible
 *
 * @param self domain participant
 * @param handle subscription handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_ignore_subscription(dds_DomainParticipant* self, dds_InstanceHandle_t handle);

/**
 * Retrieves the domain_id used to create the DomainParticipant
 * The domain_id identifies the DDS domain to which the DomainParticipant belongs. each DDS domain represents a separate  data “communication plane” isolated from other domains
 *
 * @param self domain participant
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_DomainId_t dds_DomainParticipant_get_domain_id(dds_DomainParticipant* self);

/**
 * Manually asserts the liveliness of the DomainParticipant. This is used in combination with the LIVELINESS QoS policy to indicate to the Service that the entity remains active. This operation needs to only be used if the DomainParticipant contains DataWriter entities with the LIVELINESS set to MANUAL_BY_PARTICIPANT and it only affects the liveliness of those DataWriter entities. Otherwise, it has no effect
 *
 * @param self domain participant
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_assert_liveliness(dds_DomainParticipant* self);

/**
 * Sets a default value of the Publisher QoS policies
 *
 * @param self domain participant
 * @param qos publisher qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_set_default_publisher_qos(dds_DomainParticipant* self, const dds_PublisherQos* qos);

/**
 * Retrieves a default value of the Publisher QoS policies
 *
 * @param self domain participant
 * @param qos publisher qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_default_publisher_qos(dds_DomainParticipant* self, dds_PublisherQos* qos);

/**
 * Sets a default value of the Subscriber QoS policies
 *
 * @param self domain participant
 * @param qos subscriber qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_set_default_subscriber_qos(dds_DomainParticipant* self, const dds_SubscriberQos* qos);

/**
 * Retrieves a default value of the Subscriber QoS policies
 *
 * @param self domain participant
 * @param qos subscriber qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_default_subscriber_qos(dds_DomainParticipant* self, dds_SubscriberQos* qos);

/**
 * Sets a default value of the Topic QoS policies
 *
 * @param self domain participant
 * @param qos topic qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_set_default_topic_qos(dds_DomainParticipant* self, const dds_TopicQos* qos);

/**
 * Retrieves a default value of the Topic QoS policies
 *
 * @param self domain participant
 * @param qos topic qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_default_topic_qos(dds_DomainParticipant* self, dds_TopicQos* qos);

/**
 * Retrieves the list of DomainParticipants that have been discovered in the domain
 *
 * @param self domain participant
 * @param participant_handles remote domain participants
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_discovered_participants(dds_DomainParticipant* self, dds_InstanceHandleSeq* participant_handles);

/**
 * Retrieves information on a DomainParticipant that has been discovered on the network
 *
 * @param self domain participant
 * @param participant_data participant data
 * @param participant_handles remote domain participants
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_discovered_participant_data(dds_DomainParticipant* self, dds_ParticipantBuiltinTopicData* participant_data, dds_InstanceHandle_t participant_handle);

/**
 * Retrieves the list of Topics that have been discovered in the domain
 *
 * @param self domain participant
 * @param topic_handles topic handles
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_discovered_topics(dds_DomainParticipant* self, dds_InstanceHandleSeq* topic_handles);

/**
 * Retrieves information on a Topic that has been discovered on the network
 *
 * @param self domain participant
 * @param topic_data topic data
 * @param topic_handle topic handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_discovered_topic_data(dds_DomainParticipant* self, dds_TopicBuiltinTopicData* topic_data, dds_InstanceHandle_t topic_handle);

/**
 * Checks whether or not the given a_handle represents an Entity that was created from the DomainParticipant
 *
 * The containment applies recursively. That is, it applies both to entities (TopicDescription, Publisher, or Subscriber) created directly using the DomainParticipant as well as entities created using a contained Publisher, or Subscriber as the factory, and so forth
 *
 * @param self domain participant
 * @param a_handle entity handle
 *
 * @return entity is contained or not
 */
GURUMDDS_C_DLLAPI bool dds_DomainParticipant_contains_entity(dds_DomainParticipant* self, dds_InstanceHandle_t a_handle);

/**
 * Returns the current value of the time that the service uses
 *
 * @param self domain participant
 * @param current_time current time
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipant_get_current_time(dds_DomainParticipant* self, dds_Time_t* current_time);

/**
 * DomainParticipantFactory allow the creation and destruction of DomainParticipant objects.
 *
 * DomainParticipantFactory itself has no factory. It is a pre-existing singleton object that can be accessed by means of the get_instance class operation on the DomainParticipantFactory
 */
typedef struct _dds_DomainParticipantFactory dds_DomainParticipantFactory;

/**
 * Returns the DomainParticipantFactory singleton
 *
 * The operation is idempotent, that is, it can be called multiple times without side-effects and it will return the same DomainParticipantFactory instance
 *
 * @return DomainParticipantFactory singleton
 */
GURUMDDS_C_DLLAPI dds_DomainParticipantFactory* dds_DomainParticipantFactory_get_instance(void);

/**
 * Explicitly releases all of data, entities, and system resources that used by service
 *
 * Since This operation is not reversible, It must be called very end of application lifecycle
 */
GURUMDDS_C_DLLAPI void dds_DomainParticipantFactory_shutdown(void);

/**
 * Creates a new DomainParticipant object
 *
 * The DomainParticipant signifies that the calling application intends to join the Domain identified by the domain_id argument
 *
 * @param self domainparticipantfactory
 * @param domain_id domain id
 * @param qos domainparticipant qos
 * @param a_listener domainparticipant listener
 * @param mask domainparticipant listener mask
 *
 * @return newly created DomainParticipant
 */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_DomainParticipantFactory_create_participant(dds_DomainParticipantFactory* self, dds_DomainId_t domain_id, const dds_DomainParticipantQos* qos, const dds_DomainParticipantListener* a_listener, dds_StatusMask mask);

/**
 * Deletes an existing DomainParticipant
 *
 * This operation can only be invoked if all domain entities belonging to the participant have already been deleted. Otherwise the error PRECONDITION_NOT_MET is returned
 *
 * @param self domainparticipantfactory
 * @param a_participant domainparticipant
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantFactory_delete_participant(dds_DomainParticipantFactory* self, dds_DomainParticipant* a_participant);

/**
 * Retrieves a previously created DomainParticipant belonging to specified domain_id
 *
 * If no such DomainParticipant exists, the operation will return a 'nil' value. If multiple DomainParticipant entities belonging to that domain_id exist, then the operation will return one of them
 *
 * @param self domainparticipantfactory
 * @param domain_id domain id
 *
 * @return domainparticipant
 */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_DomainParticipantFactory_lookup_participant(dds_DomainParticipantFactory* self, dds_DomainId_t domain_id);

/**
 * Sets a default value of the DomainParticipant QoS policies which will be used for newly created DomainParticipant entities in the case where the QoS policies are defaulted in the create_participant operation
 *
 * @param self domainparticipantfactory
 * @param qos domainparticipant qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantFactory_set_default_participant_qos(dds_DomainParticipantFactory* self, const dds_DomainParticipantQos* qos);

/**
 * Retrieves the default value of the DomainParticipant QoS
 *
 * @param self domainparticipantfactory
 * @param qos domainparticipant qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantFactory_get_default_participant_qos(dds_DomainParticipantFactory* self, dds_DomainParticipantQos* qos);

/**
 * Sets the value of the DomainParticipantFactory QoS policies
 *
 * @param self domainparticipantfactory
 * @param qos domainparticipantfactory qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantFactory_set_qos(dds_DomainParticipantFactory* self, const dds_DomainParticipantFactoryQos* qos);

/**
 * Retrieves the value of the DomainParticipantFactory QoS policies
 *
 * @param self domainparticipantfactory
 * @param qos domainparticipantfactory qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DomainParticipantFactory_get_qos(dds_DomainParticipantFactory* self, dds_DomainParticipantFactoryQos* qos);

/**
 * Returns the DomainParticipant to which the TopicDescription belongs
 *
 * @param self TopicDescription
 *
 * @return DomainParticipant
 */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_TopicDescription_get_participant(dds_TopicDescription* self);

/**
 * Retrieves the type_name used to create the TopicDescription
 *
 * @param self TopicDescription
 *
 * @return type name
 */
GURUMDDS_C_DLLAPI const char* dds_TopicDescription_get_type_name(dds_TopicDescription* self);

/**
 * Retrieves the name used to create the TopicDescription
 *
 * @param self TopicDescription
 *
 * @return name
 */
GURUMDDS_C_DLLAPI const char* dds_TopicDescription_get_name(dds_TopicDescription* self);

/**
 * Sets the value of the Topic QoS policies
 *
 * @param self topic
 * @param qos topic qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Topic_set_qos(dds_Topic* self, const dds_TopicQos* qos);

/**
 * Retrieves the value of the Topic QoS policies
 *
 * @param self topic
 * @param qos topic qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Topic_get_qos(dds_Topic* self, dds_TopicQos* qos);

/**
 * Installs a Listener on the Topic. The listener will only be invoked on the changes of communication status indicated by the specified mask
 *
 * Only one listener can be attached. If a listener was already set, the operation set_listener will replace it with the new one. Consequently if the value 'nil' is passed for the listener parameter to the set_listener operation, any existing listener will be removed
 *
 * @param self topic
 * @param l topic listener
 * @param mask topic listener mask
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Topic_set_listener(dds_Topic* self, const dds_TopicListener* l, dds_StatusMask mask);

/**
 * Retrieves the existing Listener attached to the Topic
 *
 * @param self topic
 *
 * @return topic listener
 */
GURUMDDS_C_DLLAPI dds_TopicListener dds_Topic_get_listener(dds_Topic* self);

/**
 * Enables the Topic
 *
 * The enable operation is idempotent. Calling enable on an already enabled Entity returns OK and has no effect
 *
 * @param self topic
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Topic_enable(dds_Topic* self);

/**
 * Retrieves the StatusCondition associated with the Topic
 *
 * @param self topic
 *
 * @return statuscondition
 */
GURUMDDS_C_DLLAPI dds_StatusCondition* dds_Topic_get_statuscondition(dds_Topic* self);

/**
 * Retrieves the list of communication statuses in the Entity that are 'triggered'
 *
 * @param self topic
 *
 * @return statusmask
 */
GURUMDDS_C_DLLAPI dds_StatusMask dds_Topic_get_status_changes(dds_Topic* self);

/**
 * Retrieves the InstanceHandle_t that represents the Topic
 *
 * @param self topic
 *
 * @return instance handle
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_Topic_get_instance_handle(dds_Topic* self);

/** @see dds_TopicDescription_get_type_name */
GURUMDDS_C_DLLAPI const char* dds_Topic_get_type_name(dds_Topic* self);

/** @see dds_TopicDescription_get_name */
GURUMDDS_C_DLLAPI const char* dds_Topic_get_name(dds_Topic* self);

/** @see dds_TopicDescription_get_participant */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_Topic_get_participant(dds_Topic* self);

/**
 * Retrieve the INCONSISTENT_TOPIC status of the Topic
 *
 * @param self topic
 * @param a_status INCONSISTENT_TOPIC status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Topic_get_inconsistent_topic_status(dds_Topic* self, dds_InconsistentTopicStatus* a_status);

/** @see dds_Topic_get_type_name */
GURUMDDS_C_DLLAPI const char* dds_ContentFilteredTopic_get_type_name(dds_ContentFilteredTopic* self);

/** @see dds_Topic_get_name */
GURUMDDS_C_DLLAPI const char* dds_ContentFilteredTopic_get_name(dds_ContentFilteredTopic* self);

/** @see dds_Topic_get_participant */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_ContentFilteredTopic_get_participant(dds_ContentFilteredTopic* self);

/**
 * Returns the Topic associated with the ContentFilteredTopic. That is, the Topic specified when the ContentFilteredTopic was created
 *
 * @param self ContentFilteredTopic
 *
 * @return topic
 */
GURUMDDS_C_DLLAPI dds_Topic* dds_ContentFilteredTopic_get_related_topic(dds_ContentFilteredTopic* self);

/**
 * Retrieves the filter_expression associated with the ContentFilteredTopic. That is, the expression specified when the ContentFilteredTopic was created
 *
 * @param self ContentFilteredTopic
 *
 * @return filter expression
 */
GURUMDDS_C_DLLAPI const char* dds_ContentFilteredTopic_get_filter_expression(dds_ContentFilteredTopic* self);

/**
 * Returns the expression_parameters associated with the ContentFilteredTopic. That is, the parameters specified on the last successful call to set_expression_parameters, or if set_expression_parameters was never called, the parameters specified when the ContentFilteredTopic was created
 *
 * @param self ContentFilteredTopic
 * @param expression_parameters expression parameters
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_ContentFilteredTopic_get_expression_parameters(dds_ContentFilteredTopic* self, dds_StringSeq* expression_parameters);

/**
 * Changes the expression_parameters associated with the ContentFilteredTopic
 *
 * @param self ContentFilteredTopic
 * @param expression_parameters expression parameters
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_ContentFilteredTopic_set_expression_parameters(dds_ContentFilteredTopic* self, const dds_StringSeq* expression_parameters);

/** @see dds_Topic_get_type_name */
GURUMDDS_C_DLLAPI const char* dds_MultiTopic_get_type_name(dds_MultiTopic* self);

/** @see dds_Topic_get_name */
GURUMDDS_C_DLLAPI const char* dds_MultiTopic_get_name(dds_MultiTopic* self);

/** @see dds_Topic_get_participant */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_MultiTopic_get_participant(dds_MultiTopic* self);

/** @see dds_ContentFilteredTopic_get_expression_parameters */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_MultiTopic_get_expression_parameters(dds_MultiTopic* self, dds_StringSeq* expression_parameters);

/** @see dds_ContentFilteredTopic_set_expression_parameters */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_MultiTopic_set_expression_parameters(dds_MultiTopic* self, const dds_StringSeq* expression_parameters);

/**
 * Retrieves the subscription_expression associated with the MultiTopic. That is, the expression specified when the MultiTopic was created
 *
 * @param self MultiTopic
 *
 * @return subscription expression
 */
GURUMDDS_C_DLLAPI const char* dds_MultiTopic_get_subscription_expression(dds_MultiTopic* self);

/**
 * Installs a Listener on the Publisher. The listener will only be invoked on the changes of communication status indicated by the specified mask
 *
 * Only one listener can be attached. If a listener was already set, the operation set_listener will replace it with the new one. Consequently if the value 'nil' is passed for the listener parameter to the set_listener operation, any existing listener will be removed
 *
 * @param self publisher
 * @param l publisher listener
 * @param mask publisher listener mask
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_set_listener(dds_Publisher* self, const dds_PublisherListener* l, dds_StatusMask mask);

/**
 * Retrieves the PublisherListener from Publisher
 *
 * @param self publisher
 *
 * @return dds_PublisherListener
 */
GURUMDDS_C_DLLAPI dds_PublisherListener dds_Publisher_get_listener(dds_Publisher* self);

/**
 * Sets the value of the Publisher QoS policies
 *
 * @param self publisher
 * @param qos publisher qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_set_qos(dds_Publisher* self, const dds_PublisherQos* qos);

/**
 * Retrieves the value of the Publisher QoS policies
 *
 * @param self publisher
 * @param qos publisher qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_get_qos(dds_Publisher* self, dds_PublisherQos* qos);

/**
 * Enables the Publisher
 *
 * The enable operation is idempotent. Calling enable on an already enabled Entity returns OK and has no effect
 *
 * @param self publisher
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_enable(dds_Publisher* self);

/**
 * Retrieves the StatusCondition associated with the Publisher
 *
 * @param self publisher
 *
 * @return statuscondition
 */
GURUMDDS_C_DLLAPI dds_StatusCondition* dds_Publisher_get_statuscondition(dds_Publisher* self);

/**
 * Retrieves the list of communication statuses in the Entity that are 'triggered'
 *
 * @param self publisher
 *
 * @return statusmask
 */
GURUMDDS_C_DLLAPI dds_StatusMask dds_Publisher_get_status_changes(dds_Publisher* self);

/**
 * Retrieves the InstanceHandle_t that represents the Publisher
 *
 * @param self publisher
 *
 * @return instance handle
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_Publisher_get_instance_handle(dds_Publisher* self);

/**
 * Creates a DataWriter
 *
 * The returned DataWriter will be attached and belongs to the Publisher. The DataWriter returned by the create_datawriter operation will in fact be a derived class, specific to the data-type associated with the Topic
 *
 * @param self publisher
 * @param a_topic topic
 * @param qos datawriter qos
 * @param a_listener datawriter listener
 * @param mask datawriter listener mask
 *
 * @return datawriter
 */
GURUMDDS_C_DLLAPI dds_DataWriter* dds_Publisher_create_datawriter(dds_Publisher* self, const dds_Topic* a_topic, const dds_DataWriterQos* qos, const dds_DataWriterListener* a_listener, dds_StatusMask mask);

/**
 * Deletes a DataWriter that belongs to the Publisher
 *
 * The delete_datawriter operation must be called on the same Publisher object used to create the DataWriter. If delete_datawriter is called on a different Publisher, the operation will have no effect and it will return PRECONDITION_NOT_MET
 *
 * The deletion of the DataWriter will automatically unregister all instances. Depending on the settings of the WRITER_DATA_LIFECYCLE QosPolicy, the deletion of the DataWriter may also dispose all instances
 *
 * @param self publisher
 * @param a_datawriter datawriter
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_delete_datawriter(dds_Publisher* self, dds_DataWriter* a_datawriter);

/**
 * Retrieves a previously created DataWriter belonging to the Publisher that is attached to a Topic with a matching topic_name
 *
 * If no such DataWriter exists, the operation will return 'nil'. If multiple DataWriter attached to the Publisher satisfy this condition, then the operation will return one of them
 *
 * @param self publisher
 * @param topic_name topic name
 *
 * @return datawriter
 */
GURUMDDS_C_DLLAPI dds_DataWriter* dds_Publisher_lookup_datawriter(dds_Publisher* self, const char* topic_name);

/**
 * Indicates a hint to the Service that the application is about to make multiple modifications using DataWriter objects belonging to the Publisher
 *
 * The use of this operation must be matched by a corresponding call to resume_publications indicating that the set of modifications has completed
 *
 * @param self publisher
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_suspend_publications(dds_Publisher* self);

/**
 * Indicates a hint to the Service that the application has completed the multiple changes initiated by the previous suspend_publications
 *
 * The call to resume_publications must match a previous call to suspend_publications. Otherwise the operation will return the error PRECONDITION_NOT_MET
 *
 * @param self publisher
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_resume_publications(dds_Publisher* self);

/** This operation is not supported */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_begin_coherent_changes(dds_Publisher* self);

/** This operation is not supported */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_end_coherent_changes(dds_Publisher* self);

/**
 * Blocks the calling thread until either all data written by the reliable DataWriter entities is acknowledged by all matched reliable DataReader entities, or else the duration specified by the max_wait parameter elapses, whichever happens first
 *
 * A return value of OK indicates that all the samples written have been acknowledged by all reliable matched data readers. a return value of TIMEOUT indicates that max_wait elapsed before all the data was acknowledg d
 *
 * @param self publisher
 * @param max_wait timeout
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_wait_for_acknowledgments(dds_Publisher* self, const dds_Duration_t* max_wait);

/**
 * Returns the DomainParticipant to which the Publisher belongs
 *
 * @param self publisher
 *
 * @return DomainParticipant
 */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_Publisher_get_participant(dds_Publisher* self);

/**
 * Deletes all the entities that were created by means of the 'create' operations on the Publisher
 *
 * Deletes all contained DataWriter objects. The operation will return PRECONDITION_NOT_MET if the any of the contained entities is in a state where it cannot be deleted. Once delete_contained_entities returns successfully, the application may delete the Publisher knowing that it has no contained DataWriter object
 *
 * @param self publisher
 *
 * @return DomainParticipant
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_delete_contained_entities(dds_Publisher* self);

/**
 * Sets a default value of the DataWriter QoS policies which will be used for newly created DataWriter entities in the case where the QoS policies are defaulted in the create_datawriter operation
 *
 * @param self publisher
 * @param qos datawriter qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_set_default_datawriter_qos(dds_Publisher* self, const dds_DataWriterQos* qos);

/**
 * Retrieves the default value of the DataWriter QoS, that is, the QoS policies which will be used for newly created DataWriter entities in the case where the QoS policies are defaulted in the create_datawriter operation
 *
 * @param self publisher
 * @param qos datawriter qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_get_default_datawriter_qos(dds_Publisher* self, dds_DataWriterQos* qos);

/**
 * Copies the policies in the a_topic_qos to the corresponding policies in the a_datawriter_qos
 *
 * @param self publisher
 * @param a_datawriter_qos datawriter qos
 * @param a_topic_qos topic qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Publisher_copy_from_topic_qos(dds_Publisher* self, dds_DataWriterQos* a_datawriter_qos, const dds_TopicQos* a_topic_qos);

/**
 * Sets the value of the DataWriter QoS policies
 *
 * @param self datawriter
 * @param qos datawriter qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_set_qos(dds_DataWriter* self, const dds_DataWriterQos* qos);

/**
 * Retrieves the value of the DataWriter QoS policies
 *
 * @param self datawriter
 * @param qos datawriter qos
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_qos(dds_DataWriter* self, dds_DataWriterQos* qos);

/**
 * Installs a Listener on the DataWriter. The listener will only be invoked on the changes of communication status indicated by the specified mask
 *
 * Only one listener can be attached. If a listener was already set, the operation set_listener will replace it with the new one. Consequently if the value 'nil' is passed for the listener parameter to the set_listener operation, any existing listener will be removed
 *
 * @param self datawriter
 * @param l datawriter listener
 * @param mask datawriter listener mask
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_set_listener(dds_DataWriter* self, const dds_DataWriterListener* a_listener, dds_StatusMask mask);

/**
 * Retrieves the existing Listener attached to the DataWriter
 *
 * @param self datawriter
 *
 * @return datawriter listener
 */
GURUMDDS_C_DLLAPI dds_DataWriterListener dds_DataWriter_get_listener(dds_DataWriter* self);

/**
 * Enables the DataWriter
 *
 * The enable operation is idempotent. Calling enable on an already enabled Entity returns OK and has no effect
 *
 * @param self datawriter
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_enable(dds_DataWriter* self);

/**
 * Retrieves the StatusCondition associated with the DataWriter
 *
 * @param self datawriter
 *
 * @return statuscondition
 */
GURUMDDS_C_DLLAPI dds_StatusCondition* dds_DataWriter_get_statuscondition(dds_DataWriter* self);

/**
 * Retrieves the list of communication statuses in the Entity that are 'triggered'
 *
 * @param self datawriter
 *
 * @return statusmask
 */
GURUMDDS_C_DLLAPI dds_StatusMask dds_DataWriter_get_status_changes(dds_DataWriter* self);

/**
 * Retrieves the InstanceHandle_t that represents the DataWriter
 *
 * @param self datawriter
 *
 * @return instance handle
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DataWriter_get_instance_handle(dds_DataWriter* self);

/**
 * Informs the Service that the application will be modifying a particular instance
 *
 * It gives an opportunity to the Service to pre-configure itself to improve performance. It takes as a parameter an instance (to get the key value) and returns a handle that can be used in successive write or dispose operations
 *
 *
 * @param self datawriter
 * @param instance_data instance data
 *
 * @return instance handle
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DataWriter_register_instance(dds_DataWriter* self, const void* instance_data);

/**
 * Performs the same function as register_instance and can be used instead of register_instance in the cases where the application desires to specify the value for the source_timestamp
 *
 * The source_timestamp potentially affects the relative order in which readers observe events from multiple writers
 *
 * @see dds_DataWriter_register_instance
 *
 * @param self datawriter
 * @param instance_data instance data
 * @param source_timestamp source timestamp
 *
 * @return instance handle
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DataWriter_register_instance_w_timestamp(dds_DataWriter* self, const void* instance_data, const dds_Time_t* source_timestamp);

/**
 * Reverses the action of register_instance
 *
 * It should only be called on an instance that is currently registered. The operation unregister_instance should be called just once per instance, regardless of how many times register_instance was called for that instance
 *
 * This operation informs the Service that the DataWriter is not intending to modify any more of that data instance. This operation also indicates that the Service can locally remove all information regarding that instance. The application should not attempt to use the handle previously allocated to that instance after calling unregister_instance. The special value HANDLE_NIL can be used for the parameter handle. This indicates that the identity of the instance should be automatically deduced from the instance (by means of the key)
 *
 * @param self datawriter
 * @param instance_data instance data
 * @param handle instance handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_unregister_instance(dds_DataWriter* self, const void* instance_data, dds_InstanceHandle_t handle);

/**
 * Performs the same function as unregister_instance and can be used instead of unregister_instance in the cases where the application desires to specify the value for the source_timestamp
 *
 * The source_timestamp potentially affects the relative order in which readers observe events from multiple writers
 *
 * @see dds_DataWriter_unregister_instance
 *
 * @param self datawriter
 * @param instance_data instance data
 * @param handle instance handle
 * @param source_timestamp source timestamp
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_unregister_instance_w_timestamp(dds_DataWriter* self, const void* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);

/**
 * Retrieve the instance key that corresponds to an instance_handle
 *
 * The operation will only fill the fields that form the key inside the key_holder instance. This operation may return BAD_PARAMETER if the a_handle does not correspond to an existing data-object known to the DataWriter
 *
 * @param self datawriter
 * @param key_holder key holder (at least 16 bytes long)
 * @param handle instance handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_key_value(dds_DataWriter* self, void* key_holder, dds_InstanceHandle_t handle);

/**
 * Takes as a parameter an instance and returns a handle that can be used in subsequent operations that accept an instance handle as an argument
 *
 * The instance parameter is only used for the purpose of examining the fields that define the key. This operation does not register the instance in question
 *
 * @param self datawriter
 * @param instance_data instance data
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DataWriter_lookup_instance(dds_DataWriter* self, const void* instance_data);

/**
 * Writes the value of a data instance. When this operation is used, the Service will automatically supply the value of the source_timestamp that is made available to DataReader objects by means of the source_timestamp attribute
 *
 * The special value HANDLE_NIL can be used for the parameter handle. This indicates that the identity of the instance should be automatically deduced from the instance_data (by means of the key). If handle is any value other than HANDLE_NIL, then it must correspond to the value returned by register_instance when the instance (identified by its key) was registered.
 *
 * As a side effect, this operation asserts liveliness on the DataWriter itself, the Publisher and the DomainParticipant
 *
 * @param self datawriter
 * @param instance_data instance data
 * @param handle instance handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_write(dds_DataWriter* self, const void* instance_data, dds_InstanceHandle_t handle);

/**
 * Performs the same function as write except that it also provides the value for the source_timestamp that is made available to DataReader objects by means of the source_timestamp attribute inside the SampleInfo
 *
 * @see dds_DataWriter_write
 *
 * @param self datawriter
 * @param instance_data instance data
 * @param handle instance handle
 * @param source_timestamp source timestamp
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_write_w_timestamp(dds_DataWriter* self, const void* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);

/**
 * Requests the middleware to delete the data
 *
 * In general, applications are made aware of the deletion by means of operations on the DataReader objects that already knew that instance. This operation does not modify the value of the instance
 *
 * The instance parameter is passed just for the purposes of identifying the instance. When this operation is used, the Service will automatically supply the value of the source_timestamp that is made available to DataReader objects by means of the source_timestamp attribute inside the SampleInfo
 *
 * @param self datawriter
 * @param instance_data instance data
 * @param instance_handle instance handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_dispose(dds_DataWriter* self, const void* instance_data, dds_InstanceHandle_t instance_handle);

/**
 * Performs the same functions as dispose except that the application provides the value for the source_timestampthat is made available to DataReader objects by means of the source_timestamp attribute inside the SampleInfo
 *
 * @param self datawriter
 * @param instance_data instance data
 * @param instance_handle instance handle
 * @param source_timestamp source timestamp
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_dispose_w_timestamp(dds_DataWriter* self, const void* instance_data, dds_InstanceHandle_t instance_handle, const dds_Time_t* source_timestamp);

/**
 * Blocks the calling thread until either all data written by the DataWriter is acknowledged by all matched DataReader entities that have RELIABILITY QoS kind RELIABLE, or else the duration specified by the max_wait parameter elapses, whichever happens first
 *
 * A return value of OK indicates that all the samples written have been acknowledged by all reliable matched data readers; a return value of TIMEOUT indicates that max_waitelapsed before all the data was acknowledged
 *
 * @param self datawriter
 * @param max_wait timeout
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_wait_for_acknowledgments(dds_DataWriter* self, const dds_Duration_t* max_wait);

/**
 * Retrieves the LIVELINESS_LOST communication status
 *
 * @param self datawriter
 * @param status LIVELINESS_LOST communication status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_liveliness_lost_status(dds_DataWriter* self, dds_LivelinessLostStatus* status);

/**
 * Retrieves the DEADLINE_MISSED communication status
 *
 * @param self datawriter
 * @param status DEADLINE_MISSED communication status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_offered_deadline_missed_status(dds_DataWriter* self, dds_OfferedDeadlineMissedStatus* status);

/**
 * Retrieves the OFFERED_INCOMPATIBLE_QOS_STATUS communication status
 *
 * @param self datawriter
 * @param status OFFERED_INCOMPATIBLE_QOS_STATUS communication status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_offered_incompatible_qos_status(dds_DataWriter* self, dds_OfferedIncompatibleQosStatus* status);

/**
 * Retrieves the PUBLICATION_MATCHED_STATUS communication status
 *
 * @param self datawriter
 * @param status PUBLICATION_MATCHED_STATUS communication status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_publication_matched_status(dds_DataWriter* self, dds_PublicationMatchedStatus* status);

/**
 * Returns the Topic associated with the DataWriter. This is the same Topic that was used to create the DataWriter
 *
 * @param self datwriter
 *
 * @return topic
 */
GURUMDDS_C_DLLAPI dds_Topic* dds_DataWriter_get_topic(dds_DataWriter* self);

/**
 * Returns the Publisher to which the DataWriter belongs
 *
 * @param self datawriter
 *
 * @return publisher
 */
GURUMDDS_C_DLLAPI dds_Publisher* dds_DataWriter_get_publisher(dds_DataWriter* self);

/**
 * Manually asserts the liveliness of the DataWriter
 *
 * This is used in combination with the LIVELINESS QoS policy to indicate to the Service that the entity remains active. This operation need only be used if the LIVELINESS setting is either MANUAL_BY_PARTICIPANT or MANUAL_BY_TOPIC. Otherwise, it has no effect
 *
 * @param self datawriter
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_assert_liveliness(dds_DataWriter* self);

/**
 * Retrieves information on a subscription that is currently 'associated' with the DataWriter
 *
 * @param self datawriter
 * @param subscription_data subscription data
 * @param subscription_handle subscription handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_matched_subscription_data(dds_DataWriter* self, dds_SubscriptionBuiltinTopicData* subscription_data, dds_InstanceHandle_t subscription_handle);

/**
 * Retrieves the list of subscriptions currently 'associated' with the DataWriter
 *
 * @param self datawriter
 * @param subscription_handles subscription handles
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataWriter_get_matched_subscriptions(dds_DataWriter* self, dds_InstanceHandleSeq* subscription_handles);

/** @see dds_Publisher_set_listener */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_set_listener(dds_Subscriber* self, const dds_SubscriberListener* l, dds_StatusMask mask);

/** @see dds_Publisher_get_listener */
GURUMDDS_C_DLLAPI dds_SubscriberListener dds_Subscriber_get_listener(dds_Subscriber* self);

/** @see dds_Publisher_set_qos */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_set_qos(dds_Subscriber* self, const dds_SubscriberQos* qos);

/** @see dds_Publisher_get_qos */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_get_qos(dds_Subscriber* self, dds_SubscriberQos* qos);

/** @see dds_Publisher_enable */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_enable(dds_Subscriber* self);

/** @see dds_Publisher_get_statuscondition */
GURUMDDS_C_DLLAPI dds_StatusCondition* dds_Subscriber_get_statuscondition(dds_Subscriber* self);

/** @see dds_Publisher_get_status_changes*/
GURUMDDS_C_DLLAPI dds_StatusMask dds_Subscriber_get_status_changes(dds_Subscriber* self);

/** @see dds_Publisher_get_instance_handle*/
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_Subscriber_get_instance_handle(dds_Subscriber* self);

/** @see dds_Publisher_create_datawriter */
GURUMDDS_C_DLLAPI dds_DataReader* dds_Subscriber_create_datareader(dds_Subscriber* self, const dds_TopicDescription* a_topic, const dds_DataReaderQos* qos, const dds_DataReaderListener* a_listener, dds_StatusMask mask);

/** @see dds_Publisher_delete_datawriter */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_delete_datareader(dds_Subscriber* self, dds_DataReader* a_datareader);

/** @see dds_Publisher_delete_contained_entities */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_delete_contained_entities(dds_Subscriber* self);

/** @see dds_Publisher_lookup_datawriter */
GURUMDDS_C_DLLAPI dds_DataReader* dds_Subscriber_lookup_datareader(dds_Subscriber* self, const char* topic_name);

/** This operation is not supported */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_get_datareaders(dds_Subscriber* self, dds_DataReaderSeq* readers, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);

/**
 * Invokes the operation on_data_available on the DataReaderListener objects attached to contained DataReaderentities with a DATA_AVAILABLE status
 *
 * @param self subscriber
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_notify_datareaders(dds_Subscriber* self);

/** This operation is not supported */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_begin_access(dds_Subscriber* self);

/** This operation is not supported */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_end_access(dds_Subscriber* self);

/** @see dds_Publisher_get_participant */
GURUMDDS_C_DLLAPI dds_DomainParticipant* dds_Subscriber_get_participant(dds_Subscriber* self);

/** @see dds_Publisher_set_default_datawriter_qos */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_set_default_datareader_qos(dds_Subscriber* self, const dds_DataReaderQos* qos);

/** @see dds_Publisher_get_default_datawriter_qos */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_get_default_datareader_qos(dds_Subscriber* self, dds_DataReaderQos* qos);

/** @see dds_Publisher_copy_from_topic_qos */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_Subscriber_copy_from_topic_qos(dds_Subscriber* self, dds_DataReaderQos* a_datareader_qos, const dds_TopicQos* a_topic_qos);

/** The SampleInfo contains information pertaining to the associated Data value */
typedef struct {
	dds_SampleStateKind		sample_state;				///< The sample_state of the Data value
	dds_ViewStateKind		view_state;					///< The view_state of the related instance
	dds_InstanceStateKind	instance_state;				///< The instance_state of the related instance
	dds_Time_t				source_timestamp;			///< The timestamp provided by the DataWriter at the time the sample was produced
	dds_InstanceHandle_t	instance_handle;			///< The InstanceHandle of the related instance
	dds_InstanceHandle_t	publication_handle;			///< The InstanceHandle of the related publication
	int32_t					disposed_generation_count;	///< The number of times the instance had become ALIVE at the time the sample was received
	int32_t					no_writers_generation_count;///< The number of times the instance had become ALIVE at the time the sample was received
	int32_t					sample_rank;				///< Provide a preview of the samples that follow within the sequence returned by the read or take operation
	int32_t					generation_rank;			///< Provide a preview of the samples that follow within the sequence returned by the read or take operation
	int32_t					absolute_generation_rank;	///< Provides a preview of what is available within the DataReader
	bool					valid_data;					///< Flag indicates whether there is data associated with the sample
} dds_SampleInfo;

/** @see dds_DataWriter_set_qos */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_set_qos(dds_DataReader* self, const dds_DataReaderQos* qos);

/** @see dds_DataWriter_get_qos */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_qos(dds_DataReader* self, dds_DataReaderQos* qos);

/** @see dds_DataWriter_set_listener */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_set_listener(dds_DataReader* self, const dds_DataReaderListener* l, dds_StatusMask mask);

/** @see dds_DataWriter_get_listener */
GURUMDDS_C_DLLAPI dds_DataReaderListener dds_DataReader_get_listener(dds_DataReader* self);

/** @see dds_DataWriter_enable */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_enable(dds_DataReader* self);

/** @see dds_DataWriter_get_statuscondition */
GURUMDDS_C_DLLAPI dds_StatusCondition* dds_DataReader_get_statuscondition(dds_DataReader* self);

/** @see dds_DataWriter_get_status_changes*/
GURUMDDS_C_DLLAPI dds_StatusMask dds_DataReader_get_status_changes(dds_DataReader* self);

/** @see dds_DataWriter_get_instance_handle */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DataReader_get_instance_handle(dds_DataReader* self);

/**
 * Creates a ReadCondition. The returned ReadCondition will be attached and belong to the DataReader
 *
 * @param self datareader
 * @param sample_states sample states
 * @param view_states view states
 * @param instance_states instance states
 *
 * @return readcondition
 */
GURUMDDS_C_DLLAPI dds_ReadCondition* dds_DataReader_create_readcondition(dds_DataReader* self, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);

/**
 * Creates a QueryCondition. The returned QueryCondition will be attached and belong to the DataReader
 *
 * @param self datareader
 * @param sample_states sample states
 * @param view_states view states
 * @param instance_states instance states
 * @param query_expression query expression
 * @param query_parameters query parameters
 *
 * @return querycondition
 */
GURUMDDS_C_DLLAPI dds_QueryCondition* dds_DataReader_create_querycondition(dds_DataReader* self, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states, const char* query_expression, const dds_StringSeq* query_parameters);

/**
 * Deletes a ReadCondition attached to the DataReader
 *
 * Since QueryCondition specializes ReadCondition it can also be used to delete a QueryCondition. If the ReadCondition is not attached to the DataReader, the operation will return the error PRECONDITION_NOT_MET
 *
 * @param self datareader
 * @param a_condition readcondition
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_delete_readcondition(dds_DataReader* self, dds_ReadCondition* a_condition);

/**
 * Read a collection of Data values from the DataReader
 *
 * The size of the returned collection will be limited to the specified max_samples
 *
 * If the DataReader has no samples that meet the constraints, the return value will be NO_DATA
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param sample_states sample states
 * @param view_states view states
 * @param instance_states instance states
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_read(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);

/**
 * Take a collection of data-samples from the DataReader and a corresponding collection of SampleInfo structures
 *
 * The size of the returned collection will be limited to the specified max_samples
 *
 * If the DataReader has no samples that meet the constraints, the return value will be NO_DATA
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param sample_states sample states
 * @param view_states view states
 * @param instance_states instance states
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_take(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);

/**
 * Accesses via 'read' the samples that match the criteria specified in the ReadCondition
 *
 * This operation is especially useful in combination with QueryCondition to filter data samples based on the content.The specified ReadCondition must be attached to the DataReader; otherwise the operation will fail and return PRECONDITION_NOT_MET
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param a_condition readcondition
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_read_w_condition(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);

/**
 * Accesses via 'take' the samples that match the criteria specified in the ReadCondition
 *
 * This operation is especially useful in combination with QueryCondition to filter data samples based on the content.The specified ReadCondition must be attached to the DataReader; otherwise the operation will fail and return PRECONDITION_NOT_MET
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param a_condition readcondition
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_take_w_condition(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);

/**
 * Copies the next, non-previously accessed Data value from the DataReader
 *
 * The operation also copies the corresponding SampleInfo. The implied order among the samples stored in the DataReader is the same as for the read operation
 *
 * @param self datareader
 * @param data_value data
 * @param sample_info sample info
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_read_next_sample(dds_DataReader* self, void* data_value, dds_SampleInfo* sample_info);

/**
 * Copies the next, non-previously accessed Data value from the DataReader
 *
 * The operation also copies the corresponding SampleInfo. The implied order among the samples stored in the DataReader is the same as for the read operation
 *
 * @param self datareader
 * @param data_value data
 * @param sample_info sample info
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_take_next_sample(dds_DataReader* self, void* data_value, dds_SampleInfo* sample_info);

/**
 * Accesses a collection of Datavalues from the DataReader. The behavior is identical to 'read' except that all samples returned belong to the single specified instance whose handle is a_handle
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param a_handle instance handle
 * @param sample_states sample states
 * @param view_states view states
 * @param instance_states instance states
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_read_instance(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);

/**
 * Accesses a collection of Datavalues from the DataReader. The behavior is identical to 'take' except that all samples returned belong to the single specified instance whose handle is a_handle
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param a_handle instance handle
 * @param sample_states sample states
 * @param view_states view states
 * @param instance_states instance states
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_take_instance(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);

/**
 * Accesses a collection of Data values from the DataReader where all the samples belong to a single instance
 *
 * The behavior is similar to read_instance except that the actual instance is not directly specified. Rather the samples will all belong to the 'next' instance with instance_handle 'greater' than the specified previous_handle that has available samples
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param previous_handle previous instance handle
 * @param sample_states sample states
 * @param view_states view states
 * @param instance_states instance states
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_read_next_instance(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);

/**
 * Accesses a collection of Datavalues from the DataReader and 'removes' them from the DataReader
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param previous_handle previous instance handle
 * @param sample_states sample states
 * @param view_states view states
 * @param instance_states instance states
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_take_next_instance(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);

/**
 * Accesses a collection of Datavalues from the DataReader
 *
 * The behavior is identical to read_next_instance except that all samples returned satisfy the specified condition
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param previous_handle previous instance handle
 * @param a_condition readcondition
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_read_next_instance_w_condition(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);

/**
 * Accesses a collection of Datavalues from the DataReader
 *
 * The behavior is identical to take_next_instance except that all samples returned satisfy the specified condition
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sample_info sequence
 * @param max_samples max samples
 * @param previous_handle previous instance handle
 * @param a_condition readcondition
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_take_next_instance_w_condition(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);

/**
 * Returns the 'loan' (i.e. data_values, and sample_infos)
 *
 * @param self datareader
 * @param data_values data sequence
 * @param sample_infos sampleinfo sequence
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_return_loan(dds_DataReader* self, dds_DataSeq* data_values, dds_SampleInfoSeq* sample_infos);

/**
 * Retrieves the LIVELINESS_CHANGED communication status
 *
 * @param self datareader
 * @param status LIVELINESS_CHANGED status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_liveliness_changed_status(dds_DataReader* self, dds_LivelinessChangedStatus* status);

/**
 * Retrieves the REQUESTED_DEADLINE_MISSED_STATUS communication status
 *
 * @param self datareader
 * @param status REQUESTED_DEADLINE_MISSED_STATUS status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_requested_deadline_missed_status(dds_DataReader* self, dds_RequestedDeadlineMissedStatus* status);

/**
 * Retrieves the REQUESTED_INCOMPATIBLE_QOS_STATUS communication status
 *
 * @param self datareader
 * @param status REQUESTED_INCOMPATIBLE_QOS_STATUS status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_requested_incompatible_qos_status(dds_DataReader* self, dds_RequestedIncompatibleQosStatus* status);

/**
 * Retrieves the SAMPLE_LOST_STATUS communication status
 *
 * @param self datareader
 * @param status SAMPLE_LOST_STATUS status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_sample_lost_status(dds_DataReader* self, dds_SampleLostStatus* status);

/**
 * Retrieves the SAMPLE_REJECTED_STATUS communication status
 *
 * @param self datareader
 * @param status SAMPLE_REJECTED_STATUS status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_sample_rejected_status(dds_DataReader* self, dds_SampleRejectedStatus* status);

/**
 * Retrieves the SUBSCRIPTION_MATCHED_STATUS communication status
 *
 * @param self datareader
 * @param status SUBSCRIPTION_MATCHED_STATUS status
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_subscription_matched_status(dds_DataReader* self, dds_SubscriptionMatchedStatus* status);

/**
 * Returns the TopicDescription associated with the DataReader. This is the same TopicDescription that was used to create the DataReader
 *
 * @param self datareader
 *
 * @return TopicDescription
 */
GURUMDDS_C_DLLAPI dds_TopicDescription* dds_DataReader_get_topicdescription(dds_DataReader* self);

/**
 * Returns the Subscriber to which the DataReader belongs
 *
 * @param self datareader
 *
 * @return Subscriber
 */
GURUMDDS_C_DLLAPI dds_Subscriber* dds_DataReader_get_subscriber(dds_DataReader* self);

/**
 * Retrieve the instance key that corresponds to an instance_handle. The operation will only fill the fields that form the key inside the key_holder instance
 *
 * This operation may return BAD_PARAMETER if the InstanceHandle_ta_handle does not correspond to an existing data-object known to the DataReader. If the implementation is not able to check invalid handles then the result in this situation is unspecified
 *
 * @param self datareader
 * @param key_holder key holder (at leat 16 bytes long)
 * @param handle instance handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_key_value(dds_DataReader* self, void* key_holder, dds_InstanceHandle_t handle);

/**
 * Takes as a parameter an instance and returns a handle that can be used in subsequent operations that accept an instance handle as an argument
 *
 * @param self datareader
 * @param instance_data instance data
 *
 * @return instance handle
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DataReader_lookup_instance(dds_DataReader* self, const void* instance_data);

/**
 * Deletes all the entities that were created by means of the 'create' operations on the DataReader
 *
 * That is, it deletes all contained ReadCondition and QueryCondition objects. The operation will return PRECONDITION_NOT_MET if the any of the contained entities is in a state where it cannot be deleted.Once delete_contained_entities returns successfully, the application may delete the DataReader knowing that it has no contained ReadCondition and QueryCondition objects
 *
 * @param self datareader
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_delete_contained_entities(dds_DataReader* self);

/** This operation is not supoprted */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_wait_for_historical_data(dds_DataReader* self, const dds_Duration_t* max_wait);

/**
 * Retrieves information on a publication that is currently associated with the DataReader
 *
 * @param self datareader
 * @param publication_data publication information data
 * @param publication_handle publication handle
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_matched_publication_data(dds_DataReader* self, dds_PublicationBuiltinTopicData* publication_data, dds_InstanceHandle_t publication_handle);

/**
 * Retrieves the list of publications currently associated with the DataReader
 *
 * @param self datareader
 * @param publication_handles publication handles
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DataReader_get_matched_publications(dds_DataReader* self, dds_InstanceHandleSeq* publication_handles);

#define dds_TheParticipantFactory							dds_DomainParticipantFactory_get_instance()
GURUMDDS_C_DLLAPI extern dds_DomainParticipantQos			dds_PARTICIPANT_QOS_DEFAULT;
GURUMDDS_C_DLLAPI extern dds_TopicQos						dds_TOPIC_QOS_DEFAULT;
GURUMDDS_C_DLLAPI extern dds_PublisherQos					dds_PUBLISHER_QOS_DEFAULT;
GURUMDDS_C_DLLAPI extern dds_SubscriberQos					dds_SUBSCRIBER_QOS_DEFAULT;
GURUMDDS_C_DLLAPI extern dds_DataWriterQos					dds_DATAWRITER_QOS_DEFAULT;
GURUMDDS_C_DLLAPI extern dds_DataReaderQos					dds_DATAREADER_QOS_DEFAULT;
#define dds_DATAWRITER_QOS_USE_TOPIC_QOS
#define dds_DATAREADER_QOS_USE_TOPIC_QOS

typedef struct _dds_DynamicTypeBuilderFactory dds_DynamicTypeBuilderFactory;
typedef struct _dds_DynamicTypeBuilder dds_DynamicTypeBuilder;
typedef struct _dds_DynamicType dds_DynamicType;
typedef struct _dds_DynamicTypeMember dds_DynamicTypeMember;
typedef struct _dds_DynamicTypeSupport dds_DynamicTypeSupport;

typedef struct _dds_DynamicDataFactory dds_DynamicDataFactory;
typedef struct _dds_DynamicData dds_DynamicData;

typedef struct _dds_DynamicDataSeq dds_DynamicDataSeq;

typedef enum _ExtensibilityKind {
	dds_FINAL_EXTENSIBILITY_KIND,
	dds_APPENDABLE_EXTENSIBILITY_KIND,
	dds_MUTABLE_EXTENSIBILITY_KIND
} dds_ExtensibilityKind;

typedef enum _TryConstructKind {
	dds_USE_DEFAULT_TRY_CONSTRUCT_KIND,
	dds_DISCARD_TRY_CONSTRUCT_KIND,
	dds_TRIM_TRY_CONSTRUCT_KIND
} dds_TryConstructKind;

typedef uint8_t dds_TypeKind;

GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_NONE;     // 0x00;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_BOOLEAN;  // 0x01;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_BYTE;     // 0x02;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_INT16;    // 0x03;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_INT32;    // 0x04;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_INT64;    // 0x05;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_UINT16;   // 0x06;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_UINT32;   // 0x07;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_UINT64;   // 0x08;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_FLOAT32;  // 0x09;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_FLOAT64;  // 0x0a;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_FLOAT128; // 0x0b;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_INT8;     // 0x0c;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_UINT8;    // 0x0d;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_CHAR8;    // 0x10;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_CHAR16;   // 0x11;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_STRING8;  // 0x20;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_STRING16; // 0x21;

GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_ALIAS; // 0x30;

GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_ENUM;    // 0x40;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_BITMASK; // 0x41;

GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_ANNOTATION; // 0x50;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_STRUCTURE;  // 0x51;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_UNION;      // 0x52;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_BITSET;     // 0x53;

GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_SEQUENCE; // 0x60;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_ARRAY;    // 0x61;
GURUMDDS_C_DLLAPI extern const dds_TypeKind TK_MAP;      // 0x62;

typedef uint32_t dds_MemberId_t;
GURUMDDS_C_DLLAPI extern const dds_MemberId_t dds_MEMBER_ID_INVALID;

typedef struct _dds_TypeDescriptor {
	dds_TypeKind kind;
	char name[256];
	dds_DynamicType* base_type;
	dds_DynamicType* discriminator_type;
	dds_UnsignedLongSeq* bound;
	dds_DynamicType* element_type;
	dds_ExtensibilityKind extensibility_kind;
	bool is_nested;
} dds_TypeDescriptor;

typedef struct _dds_MemberDescriptor {
	char name[256];
	dds_MemberId_t id;
	dds_DynamicType* type;
	char* default_value;
	uint32_t index;
	dds_LongSeq* label;
	dds_TryConstructKind try_construct_kind;

	bool is_key;
	bool is_optional;
	bool is_must_understand;
	bool is_default_label;
} dds_MemberDescriptor;

/**
 * Overwrite the contents of this descriptor with those of another descriptor such that subsequent
 * calls to equal, passing the same argument as to this method, return true.
 * The other descriptor shall not be changed by this operation.
 *
 * @param self type descriptor
 * @param other the descriptor whose contents are to be copied
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_TypeDescriptor_copy_from(dds_TypeDescriptor* self, const dds_TypeDescriptor* other);

/**
 * Finalizes the TypeDescriptor. This operation frees allocated memories for TypeDescriptor
 *
 * @param self type descriptor
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_TypeDescriptor_finalize(dds_TypeDescriptor* self);

/**
 * Compares two type descriptors and returns if they are equal
 *
 * Two type descriptors are considered equal if and only if the values of all of the properties are equal in each of them.
 *
 * @param self type descriptor
 * @param another descriptor to compare to this one
 *
 * @return whether the two descriptors are equal or not
 */
GURUMDDS_C_DLLAPI bool dds_TypeDescriptor_equals(dds_TypeDescriptor* self, const dds_TypeDescriptor* other);

/**
 * Indicates whether the states of all of this descriptor’s properties are consistent.
 *
 * @param self type descriptor
 *
 * @return whether the descriptor is consistent or not
 */
GURUMDDS_C_DLLAPI bool dds_TypeDescriptor_is_consistent(dds_TypeDescriptor* self);


/**
 * Overwrite the contents of this descriptor with those of another descriptor such that subsequent
 * calls to equals, passing the same argument as to this method, return true.
 * The other descriptor shall not be changed by this operation.
 *
 * @param self member descriptor
 * @param other the descriptor whose contents are to be copied
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_MemberDescriptor_copy_from(dds_MemberDescriptor* self, const dds_MemberDescriptor* other);

/**
 * Finalizes the MemberDescriptor. This operation frees allocated memories for MemberDescriptor
 *
 * @param self member descriptor
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_MemberDescriptor_finalize(dds_MemberDescriptor* self);

/**
 * Compares two member descriptors and returns if they are equal
 *
 * Two member descriptors are considered equal if and only if the values of all of the properties are equal in each of them.
 *
 * @param self member descriptor
 * @param another descriptor to compare to this one
 *
 * @return whether the two descriptors are equal or not
 */
GURUMDDS_C_DLLAPI bool dds_MemberDescriptor_equals(dds_MemberDescriptor* self, const dds_MemberDescriptor* other);

/**
 * Indicates whether the states of all of this descriptor’s properties are consistent.
 *
 * @param self member descriptor
 *
 * @return whether the descriptor is consistent or not
 */
GURUMDDS_C_DLLAPI bool dds_MemberDescriptor_is_consistent(dds_MemberDescriptor* self);

/**
 * Return a DynamicTypeBuilderFactory singleton.
 *
 * The operation is idempotent, that is, it can be called multiple times without side-effects and it will return the same DynamicTypeBuilderFactory instance
 *
 * @return DynamicTypeBuilderFactory singleton
 */
GURUMDDS_C_DLLAPI dds_DynamicTypeBuilderFactory* dds_DynamicTypeBuilderFactory_get_instance();

/**
 * Reclaim any resources associated with any object(s) previously returned from get_instance.
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicTypeBuilderFactory_delete_instance();

/**
 * Retrieve a DynamicType object corresponding to the indicated primitive type kind.
 *
 * @param kind the kind of the primitive type whose representation is to be returned
 *
 * @return dynamic type object
 */
GURUMDDS_C_DLLAPI dds_DynamicType* dds_DynamicTypeBuilderFactory_get_primitive_type(dds_DynamicTypeBuilderFactory* self, dds_TypeKind kind);

/**
 * Create and return a new DynamicTypeBuilder object as described by the given type descriptor.
 *
 * This method is the conventional mechanism for creating structured, enumerated, and alias types,
 * although it can also be used to create types of other kinds.
 *
 * @param self the dynamic type builder factory
 * @param descriptor the properties of the new type to create
 *
 * @return dynamic type builder object
 */
GURUMDDS_C_DLLAPI dds_DynamicTypeBuilder* dds_DynamicTypeBuilderFactory_create_type(dds_DynamicTypeBuilderFactory* self, const dds_TypeDescriptor* descriptor);

/**
 * Create and return a new DynamicTypeBuilder object with a copy of the state of the given type.
 *
 * @param self the dynamic type builder factory
 * @param type the initial state of the new type to create
 *
 * @return dynamic type builder object
 */
GURUMDDS_C_DLLAPI dds_DynamicTypeBuilder* dds_DynamicTypeBuilderFactory_create_type_copy(dds_DynamicTypeBuilderFactory* self, const dds_DynamicType* type);

/**
 * Create and return a new DynamicType object by parsing the type description at the given URL.
 *
 * @param self the dynamic type builder factory
 * @param document_uri a URL that indicates a type description document
 * @param type_name the fully qualified name of the type to be loaded from the document that is the target of the URL
 * @param include_paths a collection of URLs to directories to be searched for additional type description documents
 *
 * @return dynamic type builder object
 */
GURUMDDS_C_DLLAPI dds_DynamicTypeBuilder* dds_DynamicTypeBuilderFactory_create_type_w_uri(dds_DynamicTypeBuilderFactory* self, const char* document_uri, const char* type_name,
                                                                                          dds_StringSeq* include_paths);

/**
 * Create and return a new DynamicType object by parsing the type description contained in the given string.
 *
 * @param self the dynamic type builder factory
 * @param document a type description document
 * @param type_name the fully qualified name of the type to be loaded from the document that is the target of the URL
 * @param include_paths a collection of URLs to directories to be searched for additional type description documents
 *
 * @return dynamic type builder object
 */
GURUMDDS_C_DLLAPI dds_DynamicTypeBuilder* dds_DynamicTypeBuilderFactory_create_type_w_document(dds_DynamicTypeBuilderFactory* self, const char* document, const char* type_name,
                                                                                               dds_StringSeq* include_paths);

/**
 * Create and return a new DynamicTypeBuilder object representing a string type.
 *
 * @param self the dynamic type builder factory
 * @param bound the maximum number of elements that may be stored in a string of the new type
 *
 * @return dynamic type builder object
 */
GURUMDDS_C_DLLAPI dds_DynamicTypeBuilder* dds_DynamicTypeBuilderFactory_create_string_type(dds_DynamicTypeBuilderFactory* self, dds_UnsignedLong bound);

/** @see dds_DynamicTypeBuilderFactory_create_string_type */
GURUMDDS_C_DLLAPI dds_DynamicTypeBuilder* dds_DynamicTypeBuilderFactory_create_wstring_type(dds_DynamicTypeBuilderFactory* self, dds_UnsignedLong bound);

/**
 * Create and return a new DynamicTypeBuilder object representing a sequence type.
 * 
 * @param self the dynamic type builder factory
 * @param element_type the type of all objects that can be stored in a sequence of the new type
 * @param bound the maximum number of elements that may be stored in a sequence of the new type
 *
 * @return dynamic type builder object
 */
GURUMDDS_C_DLLAPI dds_DynamicTypeBuilder* dds_DynamicTypeBuilderFactory_create_sequence_type(dds_DynamicTypeBuilderFactory* self, const dds_DynamicType* element_type, dds_UnsignedLong bound);

/**
 * Create and return a new DynamicTypeBuilder object representing an array type.
 *
 * @param self the dynamic type builder factory
 * @param element_type the type of all objects that can be stored in an array of the new type
 * @param bound a collection of unsigned integers, the length of which is equal to the number of dimensions in the new array type
 *
 * @return dynamic type builder object
 */
GURUMDDS_C_DLLAPI dds_DynamicTypeBuilder* dds_DynamicTypeBuilderFactory_create_array_type(dds_DynamicTypeBuilderFactory* self, const dds_DynamicType* element_type, dds_UnsignedLongSeq* bound);

/**
 * Delete the given DynamicType object, which was previously created by this factory.
 *
 * @param self the dynamic type builder factory
 * @param type the type to delete
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicTypeBuilderFactory_delete_type(dds_DynamicTypeBuilderFactory* self, dds_DynamicType* type);

/**
 * Return the kind of this type.
 * 
 * The result shall be the same as the kind indicated by the type’s descriptor property.
 *
 * @param self dynamic type builder
 *
 * @return the kind of this type
 */
GURUMDDS_C_DLLAPI dds_TypeKind dds_DynamicTypeBuilder_get_kind(dds_DynamicTypeBuilder* self);

/**
 * Returns the member that corresponds to the specified member ID, if any.
 *
 * @param self dynamic type builder
 * @param member member of the type
 * @param id id of the member
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicTypeBuilder_get_member(dds_DynamicTypeBuilder* self, dds_DynamicTypeMember* member, dds_MemberId_t id);

/**
 * Returns the member that corresponds to the specified name, if any.
 *
 * @param self dynamic type builder
 * @param member member of the type
 * @param id id of the member
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicTypeBuilder_get_member_by_name(dds_DynamicTypeBuilder* self, dds_DynamicTypeMember* member, const char* name);

/**
 * Returns the member that corresponds to the specified index, if any.
 *
 * @param self dynamic type builder
 * @param member member of the type
 * @param id id of the member
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicTypeBuilder_get_member_by_index(dds_DynamicTypeBuilder* self, dds_DynamicTypeMember* member, dds_UnsignedLong idx);

/**
 * Returns the current number of members.
 *
 * @param self dynamic type builder
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_UnsignedLong dds_DynamicTypeBuilder_get_member_count(dds_DynamicTypeBuilder* self);

/**
 * Returns the fully qualified name of the type.
 *
 * It shall be identical to the name string that is a member of the descriptor.
 *
 * @param self dynamic type builder
 *
 * @return fully qualified name of the type
 */
GURUMDDS_C_DLLAPI const char* dds_DynamicTypeBuilder_get_name(dds_DynamicTypeBuilder* self);

/**
 * Compares two dynamic type builders and returns whether they are equal
 *
 * Two dynamic type builders are considered equal if and only if the values of all of the properties are equal in each of them.
 *
 * @param self dynamic type builder 
 * @param another dynamic type builder to compare to this one
 *
 * @return whether the two dynamic type builders are equal or not
 */
GURUMDDS_C_DLLAPI bool dds_DynamicTypeBuilder_equals(dds_DynamicTypeBuilder* self, const dds_DynamicTypeBuilder* other);

/**
 * Add a member to this type
 *
 * @param self dynamic type builder
 * @param descriptor a descriptor of the new member to be added
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicTypeBuilder_add_member(dds_DynamicTypeBuilder* self, const dds_MemberDescriptor* descriptor);

/**
 * Create an immutable DynamicType object containing a snapshot of this builder’s current state.
 *
 * @param self dynamic type builder
 *
 * @ return created dynamic type object
 */
GURUMDDS_C_DLLAPI dds_DynamicType* dds_DynamicTypeBuilder_build(dds_DynamicTypeBuilder* self);

/** @see dds_DynamicTypeBuilder_get_kind */
GURUMDDS_C_DLLAPI dds_TypeKind dds_DynamicType_get_kind(dds_DynamicType* self);
/** @see dds_DynamicTypeBuilder_get_member */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicType_get_member(dds_DynamicType* self, dds_DynamicTypeMember* member, dds_MemberId_t id);
/** @see dds_DynamicTypeBuilder_get_member_by_name */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicType_get_member_by_name(dds_DynamicType* self, dds_DynamicTypeMember* member, const char* name);
/** @see dds_DynamicTypeBuilder_get_member_by_index */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicType_get_member_by_index(dds_DynamicType* self, dds_DynamicTypeMember* member, dds_UnsignedLong idx);
/** @see dds_DynamicTypeBuilder_get_member_count */
GURUMDDS_C_DLLAPI dds_UnsignedLong dds_DynamicType_get_member_count(dds_DynamicType* self);
/** @see dds_DynamicTypeBuilder_get_name */
GURUMDDS_C_DLLAPI const char* dds_DynamicType_get_name(dds_DynamicType* self);
/** @see dds_DynamicTypeBuilder_equals */
GURUMDDS_C_DLLAPI bool dds_DynamicType_equals(dds_DynamicType* self, const dds_DynamicType* other);

/**
 * Retrive descriptor of the dynamic type
 *
 * @param self dynamic type
 * @param desc type descriptor
 * 
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicType_get_descriptor(dds_DynamicType* self, dds_TypeDescriptor* desc);

/**
 * Allocate new dynamic type member
 *
 * @return newly allocated dynamic type member
 */
GURUMDDS_C_DLLAPI dds_DynamicTypeMember* dds_DynamicTypeMember_alloc();

/**
 * Free dynamic type member
 *
 * @param self dynamic type member
 */
GURUMDDS_C_DLLAPI void dds_DynamicTypeMember_free(dds_DynamicTypeMember* self);

/**
 * Retrive the descriptor of the dynamic type member
 *
 * @param self dynamic type member
 * @param descriptor member descriptor
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicTypeMember_get_descriptor(dds_DynamicTypeMember* self, dds_MemberDescriptor* descriptor);

/**
 * Compares two dynamic type members and returns whether they are equal
 *
 * Two dynamic type members are considered equal if and only if the values of all of the properties are equal in each of them.
 *
 * @param self dynamic type member
 * @param another dynamic type member to compare to this one
 *
 * @return whether the two dynamic type members are equal or not
 */
GURUMDDS_C_DLLAPI bool dds_DynamicTypeMember_equals(dds_DynamicTypeMember* self, const dds_DynamicTypeMember* other);

/**
 * Return ID of the dynamic type member
 *
 * @param self dynamic type member
 *
 * @return id of the member
 */
GURUMDDS_C_DLLAPI dds_MemberId_t dds_DynamicTypeMember_get_id(dds_DynamicTypeMember* self);

/**
 * Return name of the dynamic type member
 *
 * @param self dynamic type member
 *
 * @return name of the member
 */
GURUMDDS_C_DLLAPI const char* dds_DynamicTypeMember_get_name(dds_DynamicTypeMember* self);

/** @see dds_TypeSupport_create */
GURUMDDS_C_DLLAPI dds_DynamicTypeSupport* dds_DynamicTypeSupport_create_type_support(dds_DynamicType* type);
/** @see dds_TypeSupport_delete */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicTypeSupport_delete_type_support(dds_DynamicTypeSupport* support);
/** @see dds_TypeSupport_register_type */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicTypeSupport_register_type(dds_DynamicTypeSupport* self, dds_DomainParticipant* participant, const char* type_name);
/** @see dds_TypeSupport_get_type_name */
GURUMDDS_C_DLLAPI const char* dds_DynamicTypeSupport_get_type_name(dds_DynamicTypeSupport* self);

/**
 * Return a DynamicDataFactory singleton.
 *
 * The operation is idempotent, that is, it can be called multiple times without side-effects and it will return the same DynamicDataFactory instance
 *
 * @return DynamicDataFactory singleton
 */
GURUMDDS_C_DLLAPI dds_DynamicDataFactory* dds_DynamicDataFactory_get_instance();

/**
 * Reclaim any resources associated with any object(s) previously returned from get_instance.
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataFactory_delete_instance();

/**
 * Create and return a new data sample.
 *
 * @param self dynamic data factory
 * @param type type of the data
 *
 * @return dynamic data
 */
GURUMDDS_C_DLLAPI dds_DynamicData* dds_DynamicDataFactory_create_data(dds_DynamicDataFactory* self, const dds_DynamicType* type);

/**
 * Dispose of a data sample, reclaiming any associated resources.
 *
 * @param self dynamic data factory
 * @param data dynamic data
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataFactory_delete_data(dds_DynamicDataFactory* self, dds_DynamicData* data);

/**
 * Retrieve descriptor for each value in this object, identified by the member ID.
 *
 * @param self dynamic data
 * @param value member descriptor
 * @param id id of the member
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_descriptor(dds_DynamicData* self, dds_MemberDescriptor* value, dds_MemberId_t id);

/**
 * Compares two dynamic data and returns whether they are equal
 *
 * Two dynamic data are considered equal if and only if the values of all of the properties are equal in each of them.
 *
 * @param self dynamic data 
 * @param another dynamic data to compare to this one
 *
 * @return whether the two dynamic data are equal or not
 */
GURUMDDS_C_DLLAPI bool dds_DynamicData_equals(dds_DynamicData* self, const dds_DynamicData* other);

/**
 * Return ID of the member indicated by the given name.
 *
 * @param self dynamic data
 * @param name name of the member
 *
 * @return id of the member
 */
GURUMDDS_C_DLLAPI dds_MemberId_t dds_DynamicData_get_member_id_by_name(dds_DynamicData* self, const char* name);

/**
 * Return ID of the member indicated by the given index.
 *
 * @param self dynamic data
 * @param index index of the member
 *
 * @return id of the member
 */
GURUMDDS_C_DLLAPI dds_MemberId_t dds_DynamicData_get_member_id_at_index(dds_DynamicData* self, dds_UnsignedLong index);

/**
 * Returns the item count of the data.
 *
 * The "item count" of the data depends on the type of the object.
 *   If the object is of a collection type, return the number of elements currently in the
 *   collection. In the case of an array type, this value will always be equal to the product of
 *   the bounds of all array dimensions.
 *
 *   If the object is of a bitmask type, return the number of named flags that are currently set
 *   in the bitmask.
 *
 *   If the object is of a structure or annotation type, return the number of members in the
 *   object. This value may be different than the number of members in the corresponding
 *   DynamicType —for example, some optional members may be omitted.
 *
 *   If the object is of a union type, return the number of members in the object. This number
 *   will be two if the discriminator value selects a member and one otherwise.
 *
 *   If the object is of a primitive or enumerated type, it is atomic: return one.
 *
 * @param self dynamic data
 *
 * @return the item count of the data
 */
GURUMDDS_C_DLLAPI dds_UnsignedLong dds_DynamicData_get_item_count(dds_DynamicData* self);

/**
 * Clear all values of the data.
 *
 * The meaning of “clearing” a member depends on the type of data represented by this sample:
 *   If this sample is of an aggregated type, and the indicated member is optional, remove it.
 *   If the indicated member is not optional, set it to its default value.
 * 
 *   If this sample is of a variable-length collection type, remove the indicated element,
 *   shifting any subsequent elements to the next-lowest index.
 *
 *   If the sample is of an array type, set the indicated element to its default value.
 *
 *   If the sample is of a bitmask type, clear the indicated bit.
 *
 *   If the sample is of an enumerated type, set it to the first value of the enumerated type.
 *
 *   If the sample is of a primitive type, set it to its default value.
 *
 * @param self dynamic data
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_clear_all_values(dds_DynamicData* self);
/** @see dds_DynamicData_clear_all_values */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_clear_nonkey_values(dds_DynamicData* self);
/** @see dds_DynamicData_clear_all_values */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_clear_value(dds_DynamicData* self, dds_MemberId_t id);

/**
 * Loan to the application a DynamicData object representing a value within this sample.
 *
 * @param self dynamic data
 * @param id id of the member
 *
 * @return the member value object
 */
GURUMDDS_C_DLLAPI dds_DynamicData* dds_DynamicData_loan_value(dds_DynamicData* self, dds_MemberId_t id);

/**
 * Return the 'loan'
 *
 * @param self dynamic data
 * @param value the loaned member value objectd
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_return_loaned_value(dds_DynamicData* self, dds_DynamicData* value);

/**
 * Create and return a new data sample with the same contents as this one.
 *
 * @param self dynamic data
 *
 * @return newly created copy of the data
 */
GURUMDDS_C_DLLAPI dds_DynamicData* dds_DynamicData_clone(dds_DynamicData* self);

/**
 * Get the value of the member indicated by the given id.
 *
 * @param self dynamic data
 * @param value value of the member
 * @id id of the member
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_int8_value(dds_DynamicData* self, int8_t* value, dds_MemberId_t id);

/**
 * Set the value of the member indicated by the given id.
 *
 * @param self dynamic data
 * @id id of the member
 * @param value value of the member
 *
 * @return standard return code
 */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_int8_value(dds_DynamicData* self, dds_MemberId_t id, int8_t value);

/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_uint8_value(dds_DynamicData* self, uint8_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_uint8_value(dds_DynamicData* self, dds_MemberId_t id, uint8_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_int16_value(dds_DynamicData* self, int16_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_int16_value(dds_DynamicData* self, dds_MemberId_t id, int16_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_uint16_value(dds_DynamicData* self, uint16_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_uint16_value(dds_DynamicData* self, dds_MemberId_t id, uint16_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_int32_value(dds_DynamicData* self, int32_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_int32_value(dds_DynamicData* self, dds_MemberId_t id, int32_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_uint32_value(dds_DynamicData* self, uint32_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_uint32_value(dds_DynamicData* self, dds_MemberId_t id, uint32_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_int64_value(dds_DynamicData* self, int64_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_int64_value(dds_DynamicData* self, dds_MemberId_t id, int64_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_uint64_value(dds_DynamicData* self, uint64_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_uint64_value(dds_DynamicData* self, dds_MemberId_t id, uint64_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_float32_value(dds_DynamicData* self, float* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_float32_value(dds_DynamicData* self, dds_MemberId_t id, float value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_float64_value(dds_DynamicData* self, double* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_float64_value(dds_DynamicData* self, dds_MemberId_t id, double value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_char8_value(dds_DynamicData* self, char* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_char8_value(dds_DynamicData* self, dds_MemberId_t id, char value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_char16_value(dds_DynamicData* self, uint16_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_char16_value(dds_DynamicData* self, dds_MemberId_t id, uint16_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_byte_value(dds_DynamicData* self, uint8_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_byte_value(dds_DynamicData* self, dds_MemberId_t id, uint8_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_boolean_value(dds_DynamicData* self, bool* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_boolean_value(dds_DynamicData* self, dds_MemberId_t id, bool value);

/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_string_value(dds_DynamicData* self, char** value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_string_value(dds_DynamicData* self, dds_MemberId_t id, char* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_wstring_value(dds_DynamicData* self, uint16_t** value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_wstring_value(dds_DynamicData* self, dds_MemberId_t id, uint16_t* value);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_enum_value(dds_DynamicData* self, int64_t* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_enum_value(dds_DynamicData* self, dds_MemberId_t id, int64_t value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_complex_value(dds_DynamicData* self, dds_DynamicData** value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_complex_value(dds_DynamicData* self, dds_MemberId_t id, dds_DynamicData* value);

/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_int8_values(dds_DynamicData* self, dds_OctetSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_int8_values(dds_DynamicData* self, dds_MemberId_t id, dds_OctetSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_uint8_values(dds_DynamicData* self, dds_OctetSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_uint8_values(dds_DynamicData* self, dds_MemberId_t id, dds_OctetSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_int16_values(dds_DynamicData* self, dds_ShortSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_int16_values(dds_DynamicData* self, dds_MemberId_t id, dds_ShortSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_uint16_values(dds_DynamicData* self, dds_UnsignedShortSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_uint16_values(dds_DynamicData* self, dds_MemberId_t id, dds_UnsignedShortSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_int32_values(dds_DynamicData* self, dds_LongSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_int32_values(dds_DynamicData* self, dds_MemberId_t id, dds_LongSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_uint32_values(dds_DynamicData* self, dds_UnsignedLongSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_uint32_values(dds_DynamicData* self, dds_MemberId_t id, dds_UnsignedLongSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_int64_values(dds_DynamicData* self, dds_LongLongSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_int64_values(dds_DynamicData* self, dds_MemberId_t id, dds_LongLongSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_uint64_values(dds_DynamicData* self, dds_UnsignedLongLongSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_uint64_values(dds_DynamicData* self, dds_MemberId_t id, dds_UnsignedLongLongSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_float32_values(dds_DynamicData* self, dds_FloatSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_float32_values(dds_DynamicData* self, dds_MemberId_t id, dds_FloatSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_float64_values(dds_DynamicData* self, dds_DoubleSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_float64_values(dds_DynamicData* self, dds_MemberId_t id, dds_DoubleSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_char8_values(dds_DynamicData* self, dds_CharSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_char8_values(dds_DynamicData* self, dds_MemberId_t id, dds_CharSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_char16_values(dds_DynamicData* self, dds_WcharSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_char16_values(dds_DynamicData* self, dds_MemberId_t id, dds_WcharSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_byte_values(dds_DynamicData* self, dds_OctetSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_byte_values(dds_DynamicData* self, dds_MemberId_t id, dds_OctetSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_boolean_values(dds_DynamicData* self, dds_BooleanSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_boolean_values(dds_DynamicData* self, dds_MemberId_t id, dds_BooleanSeq* value);

/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_string_values(dds_DynamicData* self, dds_StringSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_string_values(dds_DynamicData* self, dds_MemberId_t id, dds_StringSeq* value);
/** @see dds_DynamicData_get_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_get_wstring_values(dds_DynamicData* self, dds_WstringSeq* value, dds_MemberId_t id);
/** @see dds_DynamicData_set_int8_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicData_set_wstring_values(dds_DynamicData* self, dds_MemberId_t id, dds_WstringSeq* value);

/** @see dds_DataWriter_register_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DynamicDataWriter_register_instance(dds_DataWriter* self, const dds_DynamicData* instance_data);
/** @see dds_DataWriter_register_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DynamicDataWriter_register_instance_w_timestamp(dds_DataWriter* self, const dds_DynamicData* instance_data, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_unregister_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataWriter_unregister_instance(dds_DataWriter* self, const dds_DynamicData* instance_data, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_unregister_instance_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataWriter_unregister_instance_w_timestamp(dds_DataWriter* self, const dds_DynamicData* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_write */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataWriter_write(dds_DataWriter* self, const dds_DynamicData* instance_data, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_write_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataWriter_write_w_timestamp(dds_DataWriter* self, const dds_DynamicData* instance_data, dds_InstanceHandle_t handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_dispose */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataWriter_dispose(dds_DataWriter* self, const dds_DynamicData* instance_data, dds_InstanceHandle_t instance_handle);
/** @see dds_DataWriter_dispose_w_timestamp */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataWriter_dispose_w_timestamp(dds_DataWriter* self, const dds_DynamicData* instance_data, dds_InstanceHandle_t instance_handle, const dds_Time_t* source_timestamp);
/** @see dds_DataWriter_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataWriter_get_key_value(dds_DataWriter* self, void* key_holder, dds_InstanceHandle_t handle);
/** @see dds_DataWriter_lookup_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DynamicDataWriter_lookup_instance(dds_DataWriter* self, const dds_DynamicData* instance_data);

/** @see dds_DataReader_read */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_read(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_take(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_read_w_condition(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_take_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_take_w_condition(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_read_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_read_next_sample(dds_DataReader* self, dds_DynamicData* data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_take_next_sample */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_take_next_sample(dds_DataReader* self, dds_DynamicData* data_value, dds_SampleInfo* sample_info);
/** @see dds_DataReader_read_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_read_instance(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_take_instance(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t a_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_next_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_read_next_instance(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_take_next_instance */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_take_next_instance(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, dds_SampleStateMask sample_states, dds_ViewStateMask view_states, dds_InstanceStateMask instance_states);
/** @see dds_DataReader_read_next_instance_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_read_next_instance_w_condition(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_take_next_instance_w_condition */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_take_next_instance_w_condition(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos, int32_t max_samples, dds_InstanceHandle_t previous_handle, const dds_ReadCondition* a_condition);
/** @see dds_DataReader_return_loan */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_return_loan(dds_DataReader* self, dds_DynamicDataSeq* data_values, dds_SampleInfoSeq* sample_infos);
/** @see dds_DataReader_get_key_value */
GURUMDDS_C_DLLAPI dds_ReturnCode_t dds_DynamicDataReader_get_key_value(dds_DataReader* self, dds_DynamicDataSeq* key_holder, dds_InstanceHandle_t handle);
/** @see dds_DataReader_lookup_instance */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_DynamicDataReader_lookup_instance(dds_DataReader* self, const dds_DynamicData* instance_data);

/**
 * Create InstanceHandle sequence
 *
 * @param capacity initial capacity
 *
 * @return newly createdd InstanceHandle sequence
 */
GURUMDDS_C_DLLAPI dds_InstanceHandleSeq* dds_InstanceHandleSeq_create(uint32_t capacity);

/**
 * Clone InstanceHandle sequence
 *
 * @param sequence sequence to be cloned
 *
 * @return cloned InstanceHandle sequence
 */
GURUMDDS_C_DLLAPI dds_InstanceHandleSeq* dds_InstanceHandleSeq_clone(dds_InstanceHandleSeq* sequence);

/**
 * Delete InstanceHandle sequence
 *
 * @param InstanceHandle sequence
 */
GURUMDDS_C_DLLAPI void dds_InstanceHandleSeq_delete(dds_InstanceHandleSeq* sequence);

/**
 * Get length of InstanceHandle sequence
 *
 * @param InstanceHandle sequence
 *
 * @return length of InstanceHandle sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_InstanceHandleSeq_length(dds_InstanceHandleSeq* sequence);

/**
 * Get capacity of InstanceHandle sequence
 *
 * @param InstanceHandle sequence
 *
 * @return capacity of InstanceHandle sequence
 */
GURUMDDS_C_DLLAPI uint32_t dds_InstanceHandleSeq_capacity(dds_InstanceHandleSeq* sequence);

/**
 * Add an item to InstanceHandle sequence
 *
 * @param InstanceHandle sequence
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_InstanceHandleSeq_add(dds_InstanceHandleSeq* sequence, dds_InstanceHandle_t item);

/**
 * Add an item to InstanceHandle sequence at given position
 *
 * @param InstanceHandle sequence
 * @param index index
 * @param item item
 *
 * @return item is added or not
 */
GURUMDDS_C_DLLAPI bool dds_InstanceHandleSeq_add_at(dds_InstanceHandleSeq* sequence, uint32_t index, dds_InstanceHandle_t item);

/**
 * Remove an item from InstanceHandle sequence
 *
 * @param InstanceHandle sequence
 * @param index index
 *
 * @return removed item
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_InstanceHandleSeq_remove(dds_InstanceHandleSeq* sequence, uint32_t index);

/**
 * Get an item from InstanceHandle sequence
 *
 * @param InstanceHandle sequence
 * @param index index
 *
 * @return item
 */
GURUMDDS_C_DLLAPI dds_InstanceHandle_t dds_InstanceHandleSeq_get(dds_InstanceHandleSeq* sequence, uint32_t index);

/**
 * Add an array of items to InstanceHandle sequence
 *
 * @param Instance sequence
 * @param array array of items
 * @param size size of the array
 *
 * @return items are added or not
 */
GURUMDDS_C_DLLAPI bool dds_InstanceHandleSeq_add_array(dds_InstanceHandleSeq* sequence, dds_InstanceHandle_t* array, uint32_t size);

/**
 * Get an array of items from InstanceHandle sequence
 *
 * @param InstanceHandle sequence
 * @param array array of items
 * @param offset offset
 * @param size size of the array
 *
 * @return the number of copied items
 */
GURUMDDS_C_DLLAPI uint32_t dds_InstanceHandleSeq_get_array(dds_InstanceHandleSeq* sequence, dds_InstanceHandle_t* array, uint32_t offset, uint32_t size);

/** @see dds_InstanceHandleSeq_create */
GURUMDDS_C_DLLAPI dds_QosPolicyCountSeq* dds_QosPolicyCountSeq_create(uint32_t capacity);
/** @see dds_InstanceHandleSeq_clone */
GURUMDDS_C_DLLAPI dds_QosPolicyCountSeq* dds_QosPolicyCountSeq_clone(dds_QosPolicyCountSeq* sequence);
/** @see dds_InstanceHandleSeq_delete */
GURUMDDS_C_DLLAPI void dds_QosPolicyCountSeq_delete(dds_QosPolicyCountSeq* sequence);
/** @see dds_InstanceHandleSeq_length */
GURUMDDS_C_DLLAPI uint32_t dds_QosPolicyCountSeq_length(dds_QosPolicyCountSeq* sequence);
/** @see dds_InstanceHandleSeq_capacity */
GURUMDDS_C_DLLAPI uint32_t dds_QosPolicyCountSeq_capacity(dds_QosPolicyCountSeq* sequence);
/** @see dds_InstanceHandleSeq_add */
GURUMDDS_C_DLLAPI bool dds_QosPolicyCountSeq_add(dds_QosPolicyCountSeq* sequence, dds_QosPolicyCount* item);
/** @see dds_InstanceHandleSeq_add_at */
GURUMDDS_C_DLLAPI bool dds_QosPolicyCountSeq_add_at(dds_QosPolicyCountSeq* sequence, uint32_t index, dds_QosPolicyCount* item);
/** @see dds_InstanceHandleSeq_remove */
GURUMDDS_C_DLLAPI dds_QosPolicyCount* dds_QosPolicyCountSeq_remove(dds_QosPolicyCountSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_get */
GURUMDDS_C_DLLAPI dds_QosPolicyCount* dds_QosPolicyCountSeq_get(dds_QosPolicyCountSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_add_array */
GURUMDDS_C_DLLAPI bool dds_QosPolicyCountSeq_add_array(dds_QosPolicyCountSeq* seq, dds_QosPolicyCount** array, uint32_t size);
/** @see dds_InstanceHandleSeq_get_array */
GURUMDDS_C_DLLAPI uint32_t dds_QosPolicyCountSeq_get_array(dds_QosPolicyCountSeq* seq, dds_QosPolicyCount** array, uint32_t offset, uint32_t size);

/** @see dds_InstanceHandleSeq_create */
GURUMDDS_C_DLLAPI dds_DataReaderSeq* dds_DataReaderSeq_create(uint32_t capacity);
/** @see dds_InstanceHandleSeq_clone */
GURUMDDS_C_DLLAPI dds_DataReaderSeq* dds_DataReaderSeq_clone(dds_DataReaderSeq* sequence);
/** @see dds_InstanceHandleSeq_delete */
GURUMDDS_C_DLLAPI void dds_DataReaderSeq_delete(dds_DataReaderSeq* sequence);
/** @see dds_InstanceHandleSeq_length */
GURUMDDS_C_DLLAPI uint32_t dds_DataReaderSeq_length(dds_DataReaderSeq* sequence);
/** @see dds_InstanceHandleSeq_capacity */
GURUMDDS_C_DLLAPI uint32_t dds_DataReaderSeq_capacity(dds_DataReaderSeq* sequence);
/** @see dds_InstanceHandleSeq_add */
GURUMDDS_C_DLLAPI bool dds_DataReaderSeq_add(dds_DataReaderSeq* sequence, dds_DataReader* item);
/** @see dds_InstanceHandleSeq_add_at */
GURUMDDS_C_DLLAPI bool dds_DataReaderSeq_add_at(dds_DataReaderSeq* sequence, uint32_t index, dds_DataReader* item);
/** @see dds_InstanceHandleSeq_remove */
GURUMDDS_C_DLLAPI dds_DataReader* dds_DataReaderSeq_remove(dds_DataReaderSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_get */
GURUMDDS_C_DLLAPI dds_DataReader* dds_DataReaderSeq_get(dds_DataReaderSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_add_array */
GURUMDDS_C_DLLAPI bool dds_DataReaderSeq_add_array(dds_DataReaderSeq* seq, dds_DataReader** array, uint32_t size);
/** @see dds_InstanceHandleSeq_get_array */
GURUMDDS_C_DLLAPI uint32_t dds_DataReaderSeq_get_array(dds_DataReaderSeq* seq, dds_DataReader** array, uint32_t offset, uint32_t size);

/** @see dds_InstanceHandleSeq_create */
GURUMDDS_C_DLLAPI dds_DataWriterSeq* dds_DataWriterSeq_create(uint32_t capacity);
/** @see dds_InstanceHandleSeq_clone */
GURUMDDS_C_DLLAPI dds_DataWriterSeq* dds_DataWriterSeq_clone(dds_DataWriterSeq* sequence);
/** @see dds_InstanceHandleSeq_delete */
GURUMDDS_C_DLLAPI void dds_DataWriterSeq_delete(dds_DataWriterSeq* sequence);
/** @see dds_InstanceHandleSeq_length */
GURUMDDS_C_DLLAPI uint32_t dds_DataWriterSeq_length(dds_DataWriterSeq* sequence);
/** @see dds_InstanceHandleSeq_capacity */
GURUMDDS_C_DLLAPI uint32_t dds_DataWriterSeq_capacity(dds_DataWriterSeq* sequence);
/** @see dds_InstanceHandleSeq_add */
GURUMDDS_C_DLLAPI bool dds_DataWriterSeq_add(dds_DataWriterSeq* sequence, dds_DataWriter* item);
/** @see dds_InstanceHandleSeq_add_at */
GURUMDDS_C_DLLAPI bool dds_DataWriterSeq_add_at(dds_DataWriterSeq* sequence, uint32_t index, dds_DataWriter* item);
/** @see dds_InstanceHandleSeq_remove */
GURUMDDS_C_DLLAPI dds_DataWriter* dds_DataWriterSeq_remove(dds_DataWriterSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_get */
GURUMDDS_C_DLLAPI dds_DataWriter* dds_DataWriterSeq_get(dds_DataWriterSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_add_array */
GURUMDDS_C_DLLAPI bool dds_DataWriterSeq_add_array(dds_DataWriterSeq* seq, dds_DataWriter** array, uint32_t size);
/** @see dds_InstanceHandleSeq_get_array */
GURUMDDS_C_DLLAPI uint32_t dds_DataWriterSeq_get_array(dds_DataWriterSeq* seq, dds_DataWriter** array, uint32_t offset, uint32_t size);

/** @see dds_InstanceHandleSeq_create */
GURUMDDS_C_DLLAPI dds_ConditionSeq* dds_ConditionSeq_create(uint32_t capacity);
/** @see dds_InstanceHandleSeq_clone */
GURUMDDS_C_DLLAPI dds_ConditionSeq* dds_ConditionSeq_clone(dds_ConditionSeq* sequence);
/** @see dds_InstanceHandleSeq_delete */
GURUMDDS_C_DLLAPI void dds_ConditionSeq_delete(dds_ConditionSeq* sequence);
/** @see dds_InstanceHandleSeq_length */
GURUMDDS_C_DLLAPI uint32_t dds_ConditionSeq_length(dds_ConditionSeq* sequence);
/** @see dds_InstanceHandleSeq_capacity */
GURUMDDS_C_DLLAPI uint32_t dds_ConditionSeq_capacity(dds_ConditionSeq* sequence);
/** @see dds_InstanceHandleSeq_add */
GURUMDDS_C_DLLAPI bool dds_ConditionSeq_add(dds_ConditionSeq* sequence, dds_Condition* item);
/** @see dds_InstanceHandleSeq_add_at */
GURUMDDS_C_DLLAPI bool dds_ConditionSeq_add_at(dds_ConditionSeq* sequence, uint32_t index, dds_Condition* item);
/** @see dds_InstanceHandleSeq_remove */
GURUMDDS_C_DLLAPI dds_Condition* dds_ConditionSeq_remove(dds_ConditionSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_get */
GURUMDDS_C_DLLAPI dds_Condition* dds_ConditionSeq_get(dds_ConditionSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_add_array */
GURUMDDS_C_DLLAPI bool dds_ConditionSeq_add_array(dds_ConditionSeq* seq, dds_Condition** array, uint32_t size);
/** @see dds_InstanceHandleSeq_get_array */
GURUMDDS_C_DLLAPI uint32_t dds_ConditionSeq_get_array(dds_ConditionSeq* seq, dds_Condition** array, uint32_t offset, uint32_t size);

/** @see dds_InstanceHandleSeq_create */
GURUMDDS_C_DLLAPI dds_SampleInfoSeq* dds_SampleInfoSeq_create(uint32_t capacity);
/** @see dds_InstanceHandleSeq_clone */
GURUMDDS_C_DLLAPI dds_SampleInfoSeq* dds_SampleInfoSeq_clone(dds_SampleInfoSeq* sequence);
/** @see dds_InstanceHandleSeq_delete */
GURUMDDS_C_DLLAPI void dds_SampleInfoSeq_delete(dds_SampleInfoSeq* sequence);
/** @see dds_InstanceHandleSeq_length */
GURUMDDS_C_DLLAPI uint32_t dds_SampleInfoSeq_length(dds_SampleInfoSeq* sequence);
/** @see dds_InstanceHandleSeq_capacity */
GURUMDDS_C_DLLAPI uint32_t dds_SampleInfoSeq_capacity(dds_SampleInfoSeq* sequence);
/** @see dds_InstanceHandleSeq_add */
GURUMDDS_C_DLLAPI bool dds_SampleInfoSeq_add(dds_SampleInfoSeq* sequence, dds_SampleInfo* item);
/** @see dds_InstanceHandleSeq_add_at */
GURUMDDS_C_DLLAPI bool dds_SampleInfoSeq_add_at(dds_SampleInfoSeq* sequence, uint32_t index, dds_SampleInfo* item);
/** @see dds_InstanceHandleSeq_remove */
GURUMDDS_C_DLLAPI dds_SampleInfo* dds_SampleInfoSeq_remove(dds_SampleInfoSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_get */
GURUMDDS_C_DLLAPI dds_SampleInfo* dds_SampleInfoSeq_get(dds_SampleInfoSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_add_array */
GURUMDDS_C_DLLAPI bool dds_SampleInfoSeq_add_array(dds_SampleInfoSeq* seq, dds_SampleInfo** array, uint32_t size);
/** @see dds_InstanceHandleSeq_get_array */
GURUMDDS_C_DLLAPI uint32_t dds_SampleInfoSeq_get_array(dds_SampleInfoSeq* seq, dds_SampleInfo** array, uint32_t offset, uint32_t size);

/** @see dds_InstanceHandleSeq_create */
GURUMDDS_C_DLLAPI dds_DataSeq* dds_DataSeq_create(uint32_t capacity);
/** @see dds_InstanceHandleSeq_clone */
GURUMDDS_C_DLLAPI dds_DataSeq* dds_DataSeq_clone(dds_DataSeq* sequence);
/** @see dds_InstanceHandleSeq_delete */
GURUMDDS_C_DLLAPI void dds_DataSeq_delete(dds_DataSeq* sequence);
/** @see dds_InstanceHandleSeq_length */
GURUMDDS_C_DLLAPI uint32_t dds_DataSeq_length(dds_DataSeq* sequence);
/** @see dds_InstanceHandleSeq_capacity */
GURUMDDS_C_DLLAPI uint32_t dds_DataSeq_capacity(dds_DataSeq* sequence);
/** @see dds_InstanceHandleSeq_add */
GURUMDDS_C_DLLAPI bool dds_DataSeq_add(dds_DataSeq* sequence, void* item);
/** @see dds_InstanceHandleSeq_add_at */
GURUMDDS_C_DLLAPI bool dds_DataSeq_add_at(dds_DataSeq* sequence, uint32_t index, void* item);
/** @see dds_InstanceHandleSeq_remove */
GURUMDDS_C_DLLAPI void* dds_DataSeq_remove(dds_DataSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_get */
GURUMDDS_C_DLLAPI void* dds_DataSeq_get(dds_DataSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_add_array */
GURUMDDS_C_DLLAPI bool dds_DataSeq_add_array(dds_DataSeq* seq, void** array, uint32_t size);
/** @see dds_InstanceHandleSeq_get_array */
GURUMDDS_C_DLLAPI uint32_t dds_DataSeq_get_array(dds_DataSeq* seq, void** array, uint32_t offset, uint32_t size);

/** @see dds_InstanceHandleSeq_create */
GURUMDDS_C_DLLAPI dds_DataRepresentationIdSeq* dds_DataRepresentationIdSeq_create(uint32_t capacity);
/** @see dds_InstanceHandleSeq_clone */
GURUMDDS_C_DLLAPI dds_DataRepresentationIdSeq* dds_DataRepresentationIdSeq_clone(dds_DataRepresentationIdSeq* sequence);
/** @see dds_InstanceHandleSeq_delete */
GURUMDDS_C_DLLAPI void dds_DataRepresentationIdSeq_delete(dds_DataRepresentationIdSeq* sequence);
/** @see dds_InstanceHandleSeq_length */
GURUMDDS_C_DLLAPI uint32_t dds_DataRepresentationIdSeq_length(dds_DataRepresentationIdSeq* sequence);
/** @see dds_InstanceHandleSeq_capacity */
GURUMDDS_C_DLLAPI uint32_t dds_DataRepresentationIdSeq_capacity(dds_DataRepresentationIdSeq* sequence);
/** @see dds_InstanceHandleSeq_add */
GURUMDDS_C_DLLAPI bool dds_DataRepresentationIdSeq_add(dds_DataRepresentationIdSeq* sequence, dds_DataRepresentationId_t item);
/** @see dds_InstanceHandleSeq_add_at */
GURUMDDS_C_DLLAPI bool dds_DataRepresentationIdSeq_add_at(dds_DataRepresentationIdSeq* sequence, uint32_t index, dds_DataRepresentationId_t item);
/** @see dds_InstanceHandleSeq_remove */
GURUMDDS_C_DLLAPI dds_DataRepresentationId_t dds_DataRepresentationIdSeq_remove(dds_DataRepresentationIdSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_get */
GURUMDDS_C_DLLAPI dds_DataRepresentationId_t dds_DataRepresentationIdSeq_get(dds_DataRepresentationIdSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_add_array */
GURUMDDS_C_DLLAPI bool dds_DataRepresentationIdSeq_add_array(dds_DataRepresentationIdSeq* seq, dds_DataRepresentationId_t* array, uint32_t size);
/** @see dds_InstanceHandleSeq_get_array */
GURUMDDS_C_DLLAPI uint32_t dds_DataRepresentationIdSeq_get_array(dds_DataRepresentationIdSeq* seq, dds_DataRepresentationId_t* array, uint32_t offset, uint32_t size);

/** @see dds_InstanceHandleSeq_create */
GURUMDDS_C_DLLAPI dds_DynamicDataSeq* dds_DynamicDataSeq_create(uint32_t capacity);
/** @see dds_InstanceHandleSeq_clone */
GURUMDDS_C_DLLAPI dds_DynamicDataSeq* dds_DynamicDataSeq_clone(dds_DynamicDataSeq* sequence);
/** @see dds_InstanceHandleSeq_delete */
GURUMDDS_C_DLLAPI void dds_DynamicDataSeq_delete(dds_DynamicDataSeq* sequence);
/** @see dds_InstanceHandleSeq_length */
GURUMDDS_C_DLLAPI uint32_t dds_DynamicDataSeq_length(dds_DynamicDataSeq* sequence);
/** @see dds_InstanceHandleSeq_capacity */
GURUMDDS_C_DLLAPI uint32_t dds_DynamicDataSeq_capacity(dds_DynamicDataSeq* sequence);
/** @see dds_InstanceHandleSeq_add */
GURUMDDS_C_DLLAPI bool dds_DynamicDataSeq_add(dds_DynamicDataSeq* sequence, void* item);
/** @see dds_InstanceHandleSeq_add_at */
GURUMDDS_C_DLLAPI bool dds_DynamicDataSeq_add_at(dds_DynamicDataSeq* sequence, uint32_t index, void* item);
/** @see dds_InstanceHandleSeq_remove */
GURUMDDS_C_DLLAPI void* dds_DynamicDataSeq_remove(dds_DynamicDataSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_get */
GURUMDDS_C_DLLAPI void* dds_DynamicDataSeq_get(dds_DynamicDataSeq* sequence, uint32_t index);
/** @see dds_InstanceHandleSeq_add_array */
GURUMDDS_C_DLLAPI bool dds_DynamicDataSeq_add_array(dds_DynamicDataSeq* seq, void** array, uint32_t size);
/** @see dds_InstanceHandleSeq_get_array */
GURUMDDS_C_DLLAPI uint32_t dds_DynamicDataSeq_get_array(dds_DynamicDataSeq* seq, void** array, uint32_t offset, uint32_t size);

#ifdef __cplusplus
}
#endif

#endif /* __GURUMDDS_DCPS__ */

#include "typesupport.h"
#include "dcpsx.h"
